# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Utility functions for the status app. These are mostly used by
# adagios.status.views

import pynag.Utils
import pynag.Parsers
import adagios.settings
from adagios.misc.rest import add_notification, clear_notification
import simplejson as json

from collections import defaultdict
from adagios import userdata

state = defaultdict(lambda: "unknown")
state[0] = "ok"
state[1] = "warning"
state[2] = "critical"


# A special keyword that can be added to some filters to look specifically for 'unhandled' problems.
_UNHANDLED = 'unhandled'


# This key is often passed via querystring as a means to do a 'fuzzy' search of an object.
_SEARCH_KEYWORD = 'q'

# This keyword is used to define how many rows the we should return
_LIMIT_KEYWORD = 'limit'

# This is a common querystring key to filter for hosts/services in downtime.
_IN_SCHEDULED_DOWNTIME = 'in_scheduled_downtime'


# How we map from _IN_SCHEDULED_DOWNTIME querystring to something that livestatus understands
_IN_SCHEDULED_DOWNTIME_MAP = {
    '0': {'scheduled_downtime_depth': '0'},
    '1': {'scheduled_downtime_depth__gt': '0'},
}


_DEFAULT_SERVICE_COLUMNS = [
    'host_name', 'description', 'plugin_output', 'last_check', 'host_state',
    'state', 'scheduled_downtime_depth', 'last_state_change', 'acknowledged',
    'downtimes', 'host_downtimes', 'comments_with_info',
]


_DEFAULT_HOST_COLUMNS = [
    'name', 'plugin_output', 'last_check', 'state', 'services', 'downtimes',
    'last_state_change', 'acknowledged', 'parents', 'childs',  'address',
    'comments_with_info', 'scheduled_downtime_depth', 'num_services_crit',
    'num_services_warn', 'num_services_unknown', 'num_services_ok',
    'num_services_pending', 'services_with_info', 'services_with_state',
]


# Use these fields when making a generic search for hosts
_GENERIC_SEARCH_FIELDS_HOST = [
    'name__contains', 'address__contains', 'plugin_output__contains', 'alias__contains']


# Use these fields when making a generic search for services
_GENERIC_SEARCH_FIELDS_SERVICE = [
    'host_name__contains', 'description__contains', 'plugin_output__contains', 'host_address__contains']

# Name of service description attribute
_ATTRIBUTE_DESCRIPTION = 'description'


def get_all_backends():
    # TODO: Properly support multiple instances, using split here is not a good idea
    backends = adagios.settings.livestatus_path or ''
    backends = backends.split(',')
    backends = map(lambda x: x.strip(), backends)
    return backends


def livestatus(request):
    """ Returns a new pynag.Parsers.mk_livestatus() object with authauser automatically set from request.META['remoteuser']
    """

    if request is None:
        authuser = None
    elif adagios.settings.enable_authorization and not adagios.auth.has_role(request, 'administrators') and not adagios.auth.has_role(request, 'operators'):
        authuser = request.META.get('REMOTE_USER', None)
    else:
        authuser = None
    
    backends = get_all_backends()
    # we remove the disabled backends
    if backends is not None:
        try:
            user = userdata.User(request)
            if user.disabled_backends is not None:
                backends = filter(lambda x: x not in user.disabled_backends, backends)
            clear_notification("userdata problem")
        except Exception as e:
            message = "%s: %s" % (type(e), str(e))
            add_notification(level="warning", notification_id="userdata problem", message=message)

    livestatus = pynag.Parsers.MultiSite(
        nagios_cfg_file=adagios.settings.nagios_config,
        livestatus_socket_path=adagios.settings.livestatus_path,
        authuser=authuser)
    
    for i in backends:
        livestatus.add_backend(path=i, name=i)
    
    return livestatus


def query(request, *args, **kwargs):
    """ Wrapper around pynag.Parsers.mk_livestatus().query(). Any authorization logic should be performed here. """
    l = livestatus(request)
    return l.query(*args, **kwargs)


def get_hostgroups(request, *args, **kwargs):
    """ Get a list of hostgroups from mk_livestatus
    """
    l = livestatus(request)
    return l.get_hostgroups(*args, **kwargs)


def add_statistics_to_hosts(result):
    # Add statistics to every hosts:
    for host in result:
        try:
            host['num_problems'] = host['num_services_crit'] + \
                                   host['num_services_warn'] + host['num_services_unknown']
            host['children'] = host['services_with_state']

            # We are inventing a state=3 here for hosts to mean pending host.
            # We are working around a bug where unchecked hosts are assumed to be up.
            # We need to check both last_state_change and last_check because those
            # variables behave differently depending on if host has services or a
            # check command defined.
            # TODO: Remove this hack from here and into the frontend.
            if host.get('last_state_change') == 0 and host.get('last_check') == 0:
                host['state'] = 3
            host['status'] = state[host['state']]

            ok = host.get('num_services_ok')
            warn = host.get('num_services_warn')
            crit = host.get('num_services_crit')
            pending = host.get('num_services_pending')
            unknown = host.get('num_services_unknown')
            total = ok + warn + crit + pending + unknown
            host['total'] = total
            host['problems'] = warn + crit + unknown
            try:
                total = float(total)
                host['health'] = float(ok) / total * 100.0
                host['percent_ok'] = ok / total * 100
                host['percent_warn'] = warn / total * 100
                host['percent_crit'] = crit / total * 100
                host['percent_unknown'] = unknown / total * 100
                host['percent_pending'] = pending / total * 100
            except ZeroDivisionError:
                host['health'] = 'n/a'
        except Exception:
            pass


def _search_multiple_attributes(search_query, attributes, value):
    """Adds search filter to search_query that allows searching for multiple attributes.

    Args:
        search_query: pynag.Parsers.LivestatusQuery object. Filters will be adde to this search_query.
        attributes: List of strings. All the fields we want to search in.
        value: String. Value we want to search for.
    """
    for attribute in attributes:
        search_query.add_filter(attribute, value)
    if len(attributes) > 1:
        search_query.add_or_statement(len(attributes))


def _process_querystring_for_host(*args, **kwargs):
    """Take querystring from django and turn into a livestatus query.

    Note:
        This functions does preproccessing of querystring and turns
        it into a nice livestatus query. We support a few extra keywords
        for example:

        _SEARCH_KEYWORD will be turned into a fuzzy search
        _UNHANDLED keyword will make the query only look at unhandled host problems
        _IN_SCHEDULED_DOWNTIME keyword will look only for objects in scheduled downtime


    Args:
        *args: List of strings. Will be passed directly to LivestatusQuery
        **kwargs: {'str':['str']}. Querystring arguments, will be transformed
            into a valid livestatus query.

    Returns:
        pynag.Parsers.LivestatusQuery a valid livestatus query.
    """
    # By default the search dialog in adagios does a 'fuzzy search' of several fields
    # Lets extract it from our query, so we don't confuse livestatus.
    search_parameter = kwargs.pop(_SEARCH_KEYWORD, None)

    # If the unhandled keyword appears in our querystring, we automatically add
    # some search filters:
    if kwargs.pop(_UNHANDLED, False):
        kwargs.update(adagios.settings.UNHANDLED_HOSTS)

    # If _IN_SCHEDULED_DOWNTIME querystring is applied, we have to transmute it
    # To a different query that livestatus understands:
    in_scheduled_downtime = kwargs.pop(_IN_SCHEDULED_DOWNTIME, False)
    if in_scheduled_downtime:
        if isinstance(in_scheduled_downtime, list):
            in_scheduled_downtime = in_scheduled_downtime[0]
        kwargs.update(_IN_SCHEDULED_DOWNTIME_MAP[in_scheduled_downtime])

    # There are some views out there that look for both hosts and services.
    # Lets purposefully ignore attributes that we know exist for services only
    kwargs.pop(_ATTRIBUTE_DESCRIPTION, None)

    if 'host_state' in kwargs:
        kwargs['state'] = kwargs.pop('host_state')

    query = pynag.Parsers.LivestatusQuery('GET hosts', *args, **kwargs)

    if search_parameter:
        if isinstance(search_parameter, list):
            search_parameter = search_parameter[0]
        _search_multiple_attributes(query, _GENERIC_SEARCH_FIELDS_HOST, search_parameter)

    return query


def _get_limit_from_kwargs(kwargs):
    """Remove _LIMIT from kwargs if present, and convert to int.

    Returns:
        Integer. 'limit' if it was inside kwargs, otherwise adagios.settings.livestatus_limit
    """
    limit = kwargs.pop(_LIMIT_KEYWORD, adagios.settings.livestatus_limit)
    if isinstance(limit, list):
        limit = limit[0]
    limit = int(limit)
    return limit


def get_hosts(request, fields=None, *args, **kwargs):
    """ Get a list of hosts from mk_livestatus

     This is a wrapper around pynag.Parsers.mk_livestatus().query()

     Arguments:
        request: Django request object
        fields: List of strings. Which fields should be present in our response. If undefined, use _DEFAULT_HOST_FIELDS

        *args will be passed directly to livestatus
        **kwargs passed in will be converted to livestatus 'Filter:' strings

    Returns:
        List of dicts. The output from livestatus query.
    """
    limit = _get_limit_from_kwargs(kwargs)
    query = _process_querystring_for_host(*args, **kwargs)
    if limit:
        query.set_limit(limit)
    else:
        query.remove_limit()

    if fields is None:
        fields = _DEFAULT_HOST_COLUMNS

    # fields should be a list, lets create a Column: query for livestatus
    if isinstance(fields, (str, unicode)):
        fields = fields.split(',')

    query.set_columns(*fields)
    l = livestatus(request)
    hosts = l.query(query)

    add_statistics_to_hosts(hosts)
    
    hosts.sort(reverse=True, cmp=lambda a, b: cmp(a.get('num_problems'), b.get('num_problems')))
    hosts.sort(reverse=True, cmp=lambda a, b: cmp(a.get('state'), b.get('state')))

    if limit:
        return hosts[:limit]
    return hosts


def _process_querystring_for_service(*args, **kwargs):
    """Take querystring from django and turn into a livestatus query.

    Note:
        This functions does preproccessing of querystring and turns
        it into a nice livestatus query. We support a few extra keywords
        for example:

        _SEARCH_KEYWORD will be turned into a fuzzy search
        _UNHANDLED keyword will make the query only look at unhandled service
        _IN_SCHEDULED_DOWNTIME keyword will look only for objects in scheduled downtime


    Args:
        *args: List of strings. Will be passed directly to LivestatusQuery
        **kwargs: {'str':['str']}. Querystring arguments, will be transformed
            into a valid livestatus query.

    Returns:
        pynag.Parsers.LivestatusQuery a valid livestatus query.
    """

    # By default the search dialog in adagios does a 'fuzzy search' of several fields
    # Lets extract it from our query, so we don't confuse livestatus.
    search_parameter = kwargs.pop(_SEARCH_KEYWORD, None)

    # If the unhandled keyword appears in our querystring, we automatically add
    # some search filters:
    if kwargs.pop(_UNHANDLED, False):
        kwargs.update(adagios.settings.UNHANDLED_SERVICES)

    # If _IN_SCHEDULED_DOWNTIME querystring is applied, we have to transmute it
    # To a different query that livestatus understands:
    in_scheduled_downtime = kwargs.pop(_IN_SCHEDULED_DOWNTIME, False)
    if in_scheduled_downtime:
        if isinstance(in_scheduled_downtime, list):
            in_scheduled_downtime = in_scheduled_downtime[0]
        kwargs.update(_IN_SCHEDULED_DOWNTIME_MAP[in_scheduled_downtime])

    query = pynag.Parsers.LivestatusQuery('GET services', *args, **kwargs)

    if search_parameter:
        if isinstance(search_parameter, list):
            search_parameter = search_parameter[0]
        search_parameter = search_parameter.encode('utf-8')
        query.add_filters(host_name__contains=search_parameter)
        query.add_filters(description__contains=search_parameter)
        query.add_filters(plugin_output__contains=search_parameter)
        query.add_filters(host_address__contains=search_parameter)
        query.add_header_line('Or: 4')

    return query


def get_services(request=None, fields=None, *args, **kwargs):
    """ Get a list of services from Livestatus.

        This is a wrapper around pynag.Parsers.Livestatus.query()

        Arguments:
            requests: Django request object.
            fields: List of strings. If defined, collect these specific columns from livestatus. Otherwise a
              sane default from _DEFAULT_SERVICE_COLUMNS will be returned.
            *args will be passed directly to livestatus
            **kwargs passed in will be converted to livestatus 'Filter:' strings

        Returns:
            List of dicts. The output from livestatus query.
    """
    limit = _get_limit_from_kwargs(kwargs)
    query = _process_querystring_for_service(*args, **kwargs)
    if limit:
        query.set_limit(limit)
    else:
        query.remove_limit()

    fields = fields or _DEFAULT_SERVICE_COLUMNS
    if not isinstance(fields, list):  # HACK, we still have web rest queries that reference us like this
        fields = fields.split()
    query.set_columns(*fields)

    l = livestatus(request)
    services = l.query(query)

    # TODO: Can we get rid of this function in the future and workaround this another way ?
    _add_custom_tags_to_services(services)

    if limit:
        return services[:limit]
    return services


def _add_custom_tags_to_services(services):
    """Add custom tags like 'unhandled' or 'problem' to a list of services.

    Args:
        services. List of dict. Usually the output from a livestatus query.

    """
    # Add custom tags to our service list
    try:
        for service in services:
            # Tag the service with tags such as problems and unhandled
            service_tags = []
            if service['state'] != 0:
                service_tags.append('problem')
                service_tags.append('problems')
                if service['acknowledged'] == 0 and service['downtimes'] == [] and service['host_downtimes'] == []:
                    service_tags.append('unhandled')
                    service['unhandled'] = "unhandled"
                else:
                    service_tags.append('ishandled')
                    service['handled'] = "handled"
            elif service.get('last_state_change') == 0:
                service['state'] = 3
                service_tags.append('pending')
            else:
                service_tags.append('ok')
            if service['acknowledged'] == 1:
                service_tags.append('acknowledged')
            if service['downtimes'] != []:
                service_tags.append('downtime')
            service['tags'] = ' '.join(service_tags)
            service['status'] = state[service['state']]
    except Exception:
        pass
    return services


def get_contacts(request, *args, **kwargs):
    l = livestatus(request)
    return l.get_contacts(*args, **kwargs)


def get_contactgroups(request, *args, **kwargs):
    l = livestatus(request)
    return l.get_contactgroups(*args, **kwargs)


def get_statistics(request, *args, **kwargs):
    """ Return a list of dict. That contains various statistics from mk_livestatus (like service totals and host totals)
    """
    c = {}
    l = livestatus(request)
    arguments = pynag.Utils.grep_to_livestatus(*args, **kwargs)

    # Get service totals as an array of [ok,warn,crit,unknown]
    c['service_totals'] = l.get_services(
        'Stats: state = 0',
        'Stats: state = 1',
        'Stats: state = 2',
        'Stats: state = 3',
        *arguments
    ) or [0, 0, 0, 0]

    # Get host totals as an array of [up,down,unreachable]
    c['host_totals'] = l.get_hosts(
        'Stats: state = 0',
        'Stats: state = 1',
        'Stats: state = 2',
        *arguments
    ) or [0, 0, 0]

    # Get total number of host/ host_problems
    c['total_hosts'] = sum(c['host_totals'])
    c['total_host_problems'] = c['total_hosts'] - c['host_totals'][0]

    # Get total number of services/ service_problems
    c['total_services'] = sum(c['service_totals'])
    c['total_service_problems'] = c['total_services'] - c['service_totals'][0]

    # Calculate percentage of hosts/services that are "ok"
    try:
        c['service_totals_percent'] = map(lambda x: float(100.0 * x / c['total_services']), c['service_totals'])
    except ZeroDivisionError:
        c['service_totals_percent'] = [0, 0, 0, 0]
    try:
        c['host_totals_percent'] = map(lambda x: float(100.0 * x / c['total_hosts']), c['host_totals'])
    except ZeroDivisionError:
        c['host_totals_percent'] = [0, 0, 0, 0]
    
    unhandled_services = l.get_services(
        'Stats: state > 0',
        acknowledged=0,
        scheduled_downtime_depth=0,
        host_state=0,
        *arguments
    ) or [0]

    unhandled_hosts = l.get_hosts(
        'Stats: state = 1',
        acknowledged=0,
        scheduled_downtime_depth=0,
        *arguments
    ) or [0]

    c['unhandled_services'] = unhandled_services[0]
    c['unhandled_hosts'] = unhandled_hosts[0]

    total_unhandled_network_problems = l.get_hosts(
        'Filter: childs != ',
        'Stats: state = 1',
        acknowledged=0,
        scheduled_downtime_depth=0,
        *arguments
    ) or [0]
    c['total_unhandled_network_problems'] = total_unhandled_network_problems[0]

    tmp = l.get_hosts(
        'Filter: childs != ',
        'Stats: state >= 0',
        'Stats: state > 0',
        *arguments
    ) or [0, 0]

    c['total_network_parents'], c['total_network_problems'] = tmp
    return c


def get_log_entries(request, *args, **kwargs):
    """ Get log entries via pynag.Utils.LogFiles

    Returns:
        Log entries that matches the search query in (list of dict)
    """
    if not adagios.settings.enable_local_logs:
        return []
    log = pynag.Parsers.LogFiles(maincfg=adagios.settings.nagios_config)
    return log.get_log_entries(*args, **kwargs)


def get_state_history(request, *args, **kwargs):
    """ Get state history via pynag.Utils.LogFiles

    Returns:
        Log entries that matches the search query in (list of dict)
    """
    if not adagios.settings.enable_local_logs:
        return []
    log = pynag.Parsers.LogFiles(maincfg=adagios.settings.nagios_config)
    return log.get_state_history(*args, **kwargs)



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Tomas Edwardsson <tommi@tommi.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Methods for controlling and getting status of the Nagios daemon"""

from pynag.Control import daemon
from adagios import settings

class Daemon(daemon):
    def __init__(self):
        super(Daemon, self).__init__()

        # Mapping needed
        if settings.nagios_binary:
            self.nagios_bin = settings.nagios_binary
        if settings.nagios_config:
            self.nagios_cfg = settings.nagios_config
        if settings.nagios_init_script:
            self.nagios_init = settings.nagios_init_script
        if settings.nagios_service:
            self.service_name = settings.nagios_service

# vim: sts=4 expandtab autoindent



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import url, patterns

urlpatterns = patterns('',
                      url(r'^/test/?', 'adagios.misc.views.test'),
                      url(r'^/paste/?', 'adagios.misc.views.paste'),
                      url(r'^/?$', 'adagios.misc.views.index'),

                      url(r'^/settings/?', 'adagios.misc.views.settings'),
                      url(r'^/preferences/?', 'adagios.misc.views.preferences'),
                      url(r'^/nagios/?', 'adagios.misc.views.nagios'),
                      url(r'^/iframe/?', 'adagios.misc.views.iframe'),
                      url(r'^/gitlog/?', 'adagios.misc.views.gitlog'),
                      url(r'^/service/?', 'adagios.misc.views.nagios_service'),
                      url(r'^/pnp4nagios/?$', 'adagios.misc.views.pnp4nagios'),
                      url(r'^/pnp4nagios/edit(?P<filename>.+)$', 'adagios.misc.views.pnp4nagios_edit_template'),
                      url(r'^/mail', 'adagios.misc.views.mail'),
                      url(r'^/images/(?P<path>.+)$', 'django.views.static.serve', {'document_root': '/usr/share/nagios3/htdocs/images/logos/'}, name="logo"),
                      url(r'^/images/?$', 'adagios.misc.views.icons'),
                      )



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import url, patterns

urlpatterns = patterns('adagios',
                      url(r'^/?$', 'bi.views.index'),
                      url(r'^/add/?$', 'bi.views.add'),
                      url(r'^/add/subprocess/?$', 'bi.views.add_subprocess'),
                      url(r'^/add/graph/?$', 'bi.views.add_graph'),
                      url(r'^/(?P<process_name>.+)/edit/status_method$', 'bi.views.change_status_calculation_method'),
                      url(r'^/edit/(?P<process_type>.+?)/(?P<process_name>.+?)/?$', 'bi.views.edit'),
                      url(r'^/json/(?P<process_type>.+?)/(?P<process_name>.+?)/?$', 'bi.views.json'),
                      url(r'^/graphs/(?P<process_type>.+?)/(?P<process_name>.+?)/?$', 'bi.views.graphs_json'),
                      url(r'^/delete/(?P<process_type>.+?)/(?P<process_name>.+?)/?$', 'bi.views.delete'),
                      url(r'^/view/(?P<process_type>.+?)/(?P<process_name>.+?)/?$', 'bi.views.view'),
                      #(r'^/view/(?P<process_name>.+)/?$', 'bi.views.view'),
                       )




# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""

Convenient stateless functions for the status module. These are meant for programs to interact
with status of Nagios.

"""

import time
import pynag.Control.Command
import pynag.Model
import pynag.Utils
import adagios.status.utils
import pynag.Parsers
import collections

from django.utils.translation import ugettext as _
from adagios import userdata

def hosts(request, fields=None, **kwargs):
    """ Get List of hosts. Any parameters will be passed straight throught to pynag.Utils.grep()

        Arguments:
            fields -- If specified, a list of attributes to return. If unspecified, all fields are returned.

            Any **kwargs will be treated as a pynag.Utils.grep()-style filter
    """
    return adagios.status.utils.get_hosts(request=request, fields=fields, **kwargs)


def services(request, fields=None, **kwargs):
    """ Similar to hosts(), is a wrapper around adagios.status.utils.get_services()
    """
    return adagios.status.utils.get_services(request=request, fields=fields, **kwargs)

def services_dt(request, fields=None, **kwargs):
    """ Similar to hosts(), is a wrapper around adagios.status.utils.get_services()
    """
    services = adagios.status.utils.get_services(request=request, fields='host_name,description')

    result = {
        'sEcho': len(services),
	'iTotalRecords': len(services),
        'aaData': []
    }
    for service in services:
        result['aaData'].append(service.values())
    return result


def contacts(request, fields=None, *args, **kwargs):
    """ Wrapper around pynag.Parsers.mk_livestatus.get_contacts()
    """
    l = adagios.status.utils.livestatus(request)
    return l.get_contacts(*args, **kwargs)


def emails(request, *args, **kwargs):
    """ Returns a list of all emails of all contacts
    """
    l = adagios.status.utils.livestatus(request)
    return map(lambda x: x['email'], l.get_contacts('Filter: email !='))


def acknowledge_many(hostlist, servicelist, sticky=1, notify=1, persistent=0, author="adagios", comment="acknowledged by Adagios"):
    """ Same as acknowledge, but for acknowledge on many hosts services at a time.

     Arguments:
        hostlist    -- string in the format of host1;host2;host3
        servicelist -- string in the format of host1,service1;host2,service2
    """
    items = []
    for i in hostlist.split(';'):
        if not i: continue
        items.append((i, None))

    for i in servicelist.split(';'):
        if not i: continue
        host_name,service_description = i.split(',')
        items.append((host_name, service_description))
    for i in items:
        acknowledge(
                host_name=i[0],
                service_description=i[1],
                sticky=sticky,
                notify=notify,
                persistent=persistent,
                author=author,
                comment=comment
        )
    return _("Success")


def acknowledge(host_name, service_description=None, sticky=1, notify=1, persistent=0, author='adagios', comment='acknowledged by Adagios'):
    """ Acknowledge one single host or service check """
    if service_description in (None, '', u'', '_HOST_'):
        pynag.Control.Command.acknowledge_host_problem(host_name=host_name,
                                                       sticky=sticky,
                                                       notify=notify,
                                                       persistent=persistent,
                                                       author=author,
                                                       comment=comment,
                                                       )
    else:
        pynag.Control.Command.acknowledge_svc_problem(host_name=host_name,
                                                      service_description=service_description,
                                                      sticky=sticky,
                                                      notify=notify,
                                                      persistent=persistent,
                                                      author=author,
                                                      comment=comment,
                                                      )


def downtime_many(hostlist, servicelist, hostgrouplist, start_time=None, end_time=None, fixed=1, trigger_id=0, duration=7200, author='adagios', comment='Downtime scheduled by adagios', all_services_on_host=False, hostgroup_name=None):
    """ Same as downtime, but for acknowledge on many hosts services at a time.

     Arguments:
        hostlist    -- string in the format of host1;host2;host3
        hostgrouplist -- string in the format of hostgroup1;hostgroup2;hostgroup3
        servicelist -- string in the format of host1,service1;host2,service2
    """
    items = []
    for i in hostlist.split(';'):
        if not i: continue
        items.append((i, None, None))
    for i in hostgrouplist.split(';'):
        if not i: continue
        items.append((None, None, i))

    for i in servicelist.split(';'):
        if not i: continue
        host_name, service_description = i.split(',')
        items.append((host_name, service_description, None))
    for i in items:
        host_name = i[0]
        service_description = i[1]
        hostgroup_name = i[2]
        downtime(
            host_name=host_name,
            service_description=service_description,
            start_time=start_time,
            end_time=end_time,
            fixed=fixed,
            trigger_id=trigger_id,
            duration=duration,
            author=author,
            comment=comment,
            all_services_on_host=all_services_on_host,
            hostgroup_name=hostgroup_name
        )


def downtime(host_name=None, service_description=None, start_time=None, end_time=None, fixed=1, trigger_id=0, duration=7200, author='adagios', comment='Downtime scheduled by adagios', all_services_on_host=False, hostgroup_name=None):
    """ Schedule downtime for a host or a service """
    if fixed in (1, '1') and start_time in (None, ''):
        start_time = time.time()
    if fixed in (1, '1') and end_time in (None, ''):
        end_time = int(start_time) + int(duration)
    if all_services_on_host == 'false':
        all_services_on_host = False
    elif all_services_on_host == 'true':
        all_services_on_host = True

    # Check if we are supposed to schedule downtime for a whole hostgroup:
    if hostgroup_name:
        result1 = pynag.Control.Command.schedule_hostgroup_host_downtime(
            hostgroup_name=hostgroup_name,
            start_time=start_time,
            end_time=end_time,
            fixed=fixed,
            trigger_id=trigger_id,
            duration=duration,
            author=author,
            comment=comment,
        ),
        result2 = pynag.Control.Command.schedule_hostgroup_svc_downtime(
            hostgroup_name=hostgroup_name,
            start_time=start_time,
            end_time=end_time,
            fixed=fixed,
            trigger_id=trigger_id,
            duration=duration,
            author=author,
            comment=comment,
        )
        return result1, result2
    # Check if we are recursively scheduling downtime for host and all its services:
    elif all_services_on_host:
        result1 = pynag.Control.Command.schedule_host_svc_downtime(
            host_name=host_name,
            start_time=start_time,
            end_time=end_time,
            fixed=fixed,
            trigger_id=trigger_id,
            duration=duration,
            author=author,
            comment=comment,
        ),
        result2 = pynag.Control.Command.schedule_host_downtime(
            host_name=host_name,
            start_time=start_time,
            end_time=end_time,
            fixed=fixed,
            trigger_id=trigger_id,
            duration=duration,
            author=author,
            comment=comment,
        )
        return result1, result2
    # Otherwise, if this is a host
    elif service_description in (None, '', u'', '_HOST_'):
        return pynag.Control.Command.schedule_host_downtime(
            host_name=host_name,
            start_time=start_time,
            end_time=end_time,
            fixed=fixed,
            trigger_id=trigger_id,
            duration=duration,
            author=author,
            comment=comment,
        )
    # otherwise it must be a service:
    else:
        return pynag.Control.Command.schedule_svc_downtime(
            host_name=host_name,
            service_description=service_description,
            start_time=start_time,
            end_time=end_time,
            fixed=fixed,
            trigger_id=trigger_id,
            duration=duration,
            author=author,
            comment=comment,
        )

import adagios.utils


def reschedule_many(request, hostlist, servicelist, check_time=None, **kwargs):
    """ Same as reschedule() but takes a list of hosts/services as input

    Arguments:
      hostlist    -- semicolon seperated list of hosts to schedule checks for. Same as multiple calls with host_name=
      servicelist -- Same as hostlist but for services. Format is: host_name,service_description;host_name,service_description
    """
    #task = adagios.utils.Task()
    #WaitCondition = "last_check > %s" % int(time.time()- 1)
    for i in hostlist.split(';'):
        if not i: continue
        reschedule(request, host_name=i, service_description=None,
                   check_time=check_time)
        #task.add(wait, 'hosts', i, WaitCondition)
    for i in servicelist.split(';'):
        if not i: continue
        host_name,service_description = i.split(',')
        reschedule(request, host_name=host_name,
                   service_description=service_description,
                   check_time=check_time)
        #WaitObject = "{h};{s}".format(h=host_name, s=service_description)
        #task.add(wait, 'services', WaitObject, WaitCondition)
    return {'message': _("command sent successfully")}


def reschedule(request, host_name=None, service_description=None, check_time=None, wait=0, hostlist='', servicelist=''):
    """ Reschedule a check of this service/host

    Arguments:
      host_name -- Name of the host
      service_description -- Name of the service check. If left empty, host check will be rescheduled
      check_time -- timestamp of when to execute this check, if left empty, execute right now
      wait -- If set to 1, function will not return until check has been rescheduled
    """
    if check_time is None or check_time is '':
        check_time = time.time()

    check_time = int(check_time)

    if service_description in (None, '', u'', '_HOST_', 'undefined'):
        service_description = ""
        pynag.Control.Command.schedule_forced_host_check(
            host_name=host_name, check_time=check_time)
        if wait == "1":
            livestatus = adagios.status.utils.livestatus(request)
            livestatus.query("GET hosts",
                             "WaitObject: %s " % host_name,
                             "WaitCondition: last_check > %s" % check_time,
                             "WaitTrigger: check",
                             "Filter: host_name = %s" % host_name,
                             )
    else:
        pynag.Control.Command.schedule_forced_svc_check(
            host_name=host_name, service_description=service_description, check_time=check_time)
        if wait == "1":
            livestatus = adagios.status.utils.livestatus(request)
            livestatus.query("GET services",
                             "WaitObject: %s %s" % (
                                 host_name, service_description),
                             "WaitCondition: last_check > %s" % check_time,
                             "WaitTrigger: check",
                             "Filter: host_name = %s" % host_name,
                             )
    return "ok"


def comment(author, comment, host_name, service_description=None, persistent=1):
    """ Adds a comment to a particular service.

    If the "persistent" field is set to zero (0), the comment will be deleted the next time Nagios is restarted.
    Otherwise, the comment will persist across program restarts until it is deleted manually. """

    if service_description in (None, '', u'', '_HOST_'):
        pynag.Control.Command.add_host_comment(
            host_name=host_name, persistent=persistent, author=author, comment=comment)
    else:
        pynag.Control.Command.add_svc_comment(
            host_name=host_name, service_description=service_description, persistent=persistent, author=author, comment=comment)
    return "ok"


def delete_comment(comment_id, object_type=None, host_name=None, service_description=None):
    """
    """
    if not host_name:
        # TODO host_name is not used here, why do we need it ?
        pass
    if object_type == "host" or service_description in (None, '', u'', '_HOST_'):
        pynag.Control.Command.del_host_comment(comment_id=comment_id)
    else:
        pynag.Control.Command.del_svc_comment(comment_id=comment_id)
    return "ok"


def edit(object_type, short_name, attribute_name, new_value):
    """ Change one single attribute for one single object.

    Arguments:
      object_type    -- Type of object to change (i.e. "host","service", etc)
      short_name      -- Short Name of the object f.e. the host_name of a host
      attribute_name -- Name of attribute to change .. f.e. 'address'
      new_value      -- New value of the object .. f.e. '127.0.0.1'
    Examples:
      edit('host','localhost','address','127.0.0.1')
      edit('service', 'localhost/Ping', 'contactgroups', 'None')
    """
    # TODO : MK Livestatus access acording to remote_user
    c = pynag.Model.string_to_class[object_type]
    my_obj = c.objects.get_by_shortname(short_name)
    my_obj[attribute_name] = new_value
    my_obj.save()
    return str(my_obj)


def get_map_data(request, host_name=None):
    """ Returns a list of (host_name,2d_coords). If host_name is provided, returns a list with only that host """
    livestatus = adagios.status.utils.livestatus(request)
    all_hosts = livestatus.query('GET hosts', )
    hosts_with_coordinates = pynag.Model.Host.objects.filter(
        **{'2d_coords__exists': True})
    hosts = []
    connections = []
    for i in all_hosts:
        name = i['name']
        if host_name in (None, '', name):
            # If x does not have any coordinates, break
            coords = None
            for x in hosts_with_coordinates:
                if x.host_name == name:
                    coords = x['2d_coords']
                    break
            if coords is None:
                continue

            tmp = coords.split(',')
            if len(tmp) != 2:
                continue

            x, y = tmp
            host = {}
            host['host_name'] = name
            host['state'] = i['state']
            i['x_coordinates'] = x
            i['y_coordinates'] = y

            hosts.append(i)

    # For all hosts that have network parents, lets return a proper line for
    # those two
    for i in hosts:
        # Loop through all network parents. If network parent is also in our hostlist
        # Then create a connection between the two
        for parent in i.get('parents'):
            for x in hosts:
                if x.get('name') == parent:
                    connection = {}
                    connection['parent_x_coordinates'] = x.get('x_coordinates')
                    connection['parent_y_coordinates'] = x.get('y_coordinates')
                    connection['child_x_coordinates'] = i.get('x_coordinates')
                    connection['child_y_coordinates'] = i.get('y_coordinates')
                    connection['state'] = i.get('state')
                    connection['description'] = i.get('name')
                    connections.append(connection)
    result = {}
    result['hosts'] = hosts
    result['connections'] = connections
    return result


def change_host_coordinates(host_name, latitude, longitude):
    """ Updates longitude and latitude for one specific host """
    host = pynag.Model.Host.objects.get_by_shortname(host_name)
    coords = "%s,%s" % (latitude, longitude)
    host['2d_coords'] = coords
    host.save()


def autocomplete(request, q):
    """ Returns a list of {'hosts':[], 'hostgroups':[],'services':[]} matching search query q
    """
    if q is None:
        q = ''
    result = {}

    hosts = adagios.status.utils.get_hosts(request, host_name__contains=q)
    services = adagios.status.utils.get_services(request, service_description__contains=q)
    hostgroups = adagios.status.utils.get_hostgroups(request, hostgroup_name__contains=q)

    result['hosts'] = sorted(set(map(lambda x: x['name'], hosts)))
    result['hostgroups'] = sorted(set(map(lambda x: x['name'], hostgroups)))
    result['services'] = sorted(set(map(lambda x: x['description'], services)))

    return result


def delete_downtime(downtime_id, is_service=True):
    """ Delete one specific downtime with id that matches downtime_id.

    Arguments:
      downtime_id -- Id of the downtime to be deleted
      is_service  -- If set to True or 1, then this is assumed to be a service downtime, otherwise assume host downtime
    """
    if is_service in (True, 1, '1'):
        pynag.Control.Command.del_svc_downtime(downtime_id)
    else:
        pynag.Control.Command.del_host_downtime(downtime_id)
    return "ok"


def top_alert_producers(request, limit=5, start_time=None, end_time=None):
    """ Return a list of ["host_name",number_of_alerts]

     Arguments:
        limit      -- Limit output to top n hosts (default 5)
        start_time -- Search log starting with start_time (default since last log rotation)
    """
    if start_time == '':
        start_time = None
    if end_time == '':
        end_time = None
    log = adagios.status.utils.get_state_history(request, start_time=start_time, end_time=end_time)
    top_alert_producers = collections.defaultdict(int)
    for i in log:
        if 'host_name' in i and 'state' in i and i['state'] > 0:
            top_alert_producers[i['host_name']] += 1
    top_alert_producers = top_alert_producers.items()
    top_alert_producers.sort(cmp=lambda a, b: cmp(a[1], b[1]), reverse=True)
    if limit > len(top_alert_producers):
        top_alert_producers = top_alert_producers[:int(limit)]
    return top_alert_producers


def log_entries(request, *args, **kwargs):
    """ Same as pynag.Parsers.Logfiles().get_log_entries()

    Arguments:
   start_time -- unix timestamp. if None, return all entries from today
   end_time -- If specified, only fetch log entries older than this (unix timestamp)
   strict   -- If True, only return entries between start_time and end_time, if False,
            -- then return entries that belong to same log files as given timeset
   search   -- If provided, only return log entries that contain this string (case insensitive)
   kwargs   -- All extra arguments are provided as filter on the log entries. f.e. host_name="localhost"
    Returns:
   List of dicts

    """
    return adagios.status.utils.get_log_entries(request, *args, **kwargs)


def state_history(
        request, start_time=None, end_time=None, object_type=None, host_name=None,
        service_description=None, hostgroup_name=None):
    """ Returns a list of dicts, with the state history of hosts and services. Parameters behaves similar to get_log_entries

    """
    if start_time == '':
        start_time = None
    if end_time == '':
        end_time = None
    if host_name == '':
        host_name = None
    if service_description == '':
        service_description = None
    log_entries = adagios.status.utils.get_state_history(
        request, start_time=start_time, end_time=end_time,
        host_name=host_name, service_description=service_description)
    if object_type == 'host' or object_type == 'service':
        pass
    elif object_type == 'hostgroup':
        hg = pynag.Model.Hostgroup.objects.get_by_shortname(hostgroup_name)
        hosts = hg.get_effective_hosts()
        hostnames = map(lambda x: x.host_name, hosts)
        log_entries = filter(lambda x: x['host_name'] in hostnames, log_entries)
    else:
        raise Exception(_("Unsupported object type: %s") % object_type)

    # Add some css-hints for and duration of each state history entry as percent of duration
    # this is used by all views that have state history and on top of it a progress bar which shows
    # Up/downtime totals.
    c = {'log': log_entries }
    if len(c['log']) > 0:
        log = c['log']
        c['start_time'] = start_time = log[0]['time']
        c['end_time'] = log[-1]['time']
        now = time.time()

        total_duration = now - start_time
        css_hint = {}
        css_hint[0] = 'success'
        css_hint[1] = 'warning'
        css_hint[2] = 'danger'
        css_hint[3] = 'info'
        for i in log:
            i['duration_percent'] = 100 * i['duration'] / total_duration
            i['bootstrap_status'] = css_hint[i['state']]

    return log_entries


def _get_service_model(host_name, service_description=None):
    """ Return one pynag.Model.Service object for one specific service as seen

    from status point of view. That means it will do its best to return a service
    that was assigned to hostgroup but the caller requested a specific host.

    Returns:
        pynag.Model.Service object
    Raises:
        KeyError if not found
    """
    try:
        return pynag.Model.Service.objects.get_by_shortname("%s/%s" % (host_name, service_description))
    except KeyError, e:
        host = pynag.Model.Host.objects.get_by_shortname(host_name)
        for i in host.get_effective_services():
            if i.service_description == service_description:
                return i
        raise e


def command_line(host_name, service_description=None):
    """ Returns effective command line for a host or a service (i.e. resolves check_command) """
    try:
        obj = _get_host_or_service(host_name, service_description)
        return obj.get_effective_command_line(host_name=host_name)
    except KeyError:
        return _("Could not resolve commandline. Object not found")


def _get_host_or_service(host_name, service_description=None):
    """ Return a pynag.Model.Host or pynag.Model.Service or raise exception if none are found """
    host = pynag.Model.Host.objects.get_by_shortname(host_name)
    if not service_description or service_description == '_HOST_':
        return host
    else:
        search_result = pynag.Model.Service.objects.filter(host_name=host_name, service_description=service_description)
        if search_result:
            return search_result[0]
        # If no services were found, the service might be applied to a hostgroup
        for service in host.get_effective_services():
            if service.service_description == service_description:
                return service
    raise KeyError(_("Object not found"))


def update_check_command(host_name, service_description=None, **kwargs):
    """ Saves all custom variables of a given service
    """
    try:
        for k, v in kwargs.items():
            if service_description is None or service_description == '':
                obj = pynag.Model.Host.objects.get_by_shortname(host_name)
            else:
                obj = pynag.Model.Service.objects.get_by_shortname(
                    "%s/%s" % (host_name, service_description))
            if k.startswith("$_SERVICE") or k.startswith('$ARG') or k.startswith('$_HOST'):
                obj.set_macro(k, v)
                obj.save()
        return _("Object saved")
    except KeyError:
        raise Exception(_("Object not found"))


def get_business_process_names():
    """ Returns all configured business processes
    """
    import adagios.businessprocess
    return map(lambda x: x.name, adagios.businessprocess.get_all_processes())


def get(request, object_type, *args, **kwargs):
    livestatus_arguments = pynag.Utils.grep_to_livestatus(*args, **kwargs)
    if not object_type.endswith('s'):
        object_type += 's'
    if 'name__contains' in kwargs and object_type == 'services':
        name = str(kwargs['name__contains'])
        livestatus_arguments = filter(
            lambda x: x.startswith('name'), livestatus_arguments)
        livestatus_arguments.append('Filter: host_name ~ %s' % name)
        livestatus_arguments.append('Filter: description ~ %s' % name)
        livestatus_arguments.append('Or: 2')
    livestatus = adagios.status.utils.livestatus(request)
    results = livestatus.query('GET %s' % object_type, *livestatus_arguments)

    if object_type == 'service':
        for i in results:
            i['name'] = i.get('host_name') + "/" + i.get('description')
    return results


def get_business_process(process_name=None, process_type=None):
    """ Returns a list of all processes in json format.

    If process_name is specified, return all sub processes.
    """
    import adagios.bi
    if not process_name:
        processes = adagios.bi.get_all_processes()
    else:
        process = adagios.bi.get_business_process(str(process_name), process_type)
        processes = process.get_processes()
    result = []
    # Turn processes into nice json
    for i in processes:
        json = {}
        json['state'] = i.get_status()
        json['name'] = i.name
        json['display_name'] = i.display_name
        json['subprocess_count'] = len(i.processes)
        json['process_type'] = i.process_type
        result.append(json)
    return result


def remove_downtime(request, host_name, service_description=None, downtime_id=None):
    """ Remove downtime for one specific host or service """
    downtimes_to_remove = []
    # If downtime_id is not provided, remove all downtimes of that service or host
    if downtime_id:
        downtimes_to_remove.append(downtime_id)
    else:
        livestatus = adagios.status.utils.livestatus(request)
        query_parameters = list()
        query_parameters.append('GET downtimes')
        query_parameters.append('Filter: host_name = {host_name}'.format(**locals()))
        if service_description:
            query_parameters.append('Filter: service_description = {service_description}'.format(**locals()))
        result = livestatus.query(*query_parameters)
        for i in result:
            downtime_id = i['id']
            downtimes_to_remove.append(downtime_id)

    if service_description:
        for i in downtimes_to_remove:
            pynag.Control.Command.del_svc_downtime(downtime_id=i)
    else:
        for i in downtimes_to_remove:
            pynag.Control.Command.del_host_downtime(downtime_id=i)
    return "ok"


def remove_acknowledgement(host_name, service_description=None):
    """ Remove downtime for one specific host or service """
    if not service_description:
        pynag.Control.Command.remove_host_acknowledgement(host_name=host_name)
    else:
        pynag.Control.Command.remove_svc_acknowledgement(host_name=host_name, service_description=service_description)
    return "ok"


def submit_check_result(request, host_name, service_description=None, autocreate=False, status_code=3, plugin_output=_("No message was entered"), performance_data=""):
    """ Submit a passive check_result for a given host or a service

    Arguments:
        host_name           -- Name of the host you want to submit check results for
        service_description -- If provided, submit a result for service this service instead of a host
        autocreate          -- If this is set to True, and host/service does not exist. It will be created
        status_code              -- Nagios style status for the check (0,1,2,3 which means ok,warning,critical, etc)
        plugin_output       -- The text output of the check to display in a web interface
        performance_data    -- Optional, If there are any performance metrics to display
    """
    livestatus = adagios.status.utils.livestatus(request)
    result = {}
    output = plugin_output + " | " + performance_data
    if not service_description:
        object_type = 'host'
        args = pynag.Utils.grep_to_livestatus(host_name=host_name)
        objects = livestatus.get_hosts(*args)
    else:
        object_type = 'service'
        args = pynag.Utils.grep_to_livestatus(host_name=host_name, service_description=service_description)
        objects = livestatus.get_services(*args)

    if not objects and autocreate is True:
        raise Exception(_("Autocreate not implemented yet"))
    elif not objects:
        result['error'] = 'No %s with that name' % object_type
    else:
        if object_type == 'host':
            pynag.Control.Command.process_host_check_result(host_name, status_code, output)
        else:
            pynag.Control.Command.process_service_check_result(host_name, service_description, status_code, output)
        result['message'] = _("Command has been submitted.")
    return result


def statistics(request, **kwargs):
    """ Returns a dict with various statistics on status data. """
    return adagios.status.utils.get_statistics(request, **kwargs)


def metrics(request, **kwargs):
    """ Returns a list of dicts which contain service perfdata metrics
    """
    result = []
    fields = "host_name description perf_data state host_state".split()
    services = adagios.status.utils.get_services(request, fields=fields, **kwargs)
    for service in services:
        metrics = pynag.Utils.PerfData(service['perf_data']).metrics
        metrics = filter(lambda x: x.is_valid(), metrics)
        for metric in metrics:
            metric_dict = {
                'host_name': service['host_name'],
                'service_description': service['description'],
                'state': service['state'],
                'host_state': service['host_state'],
                'label': metric.label,
                'value': metric.value,
                'uom': metric.uom,
                'warn': metric.warn,
                'crit': metric.crit,
                'min': metric.min,
                'max': metric.max,
            }
            result.append(metric_dict)
    return result

def metric_names(request, **kwargs):
    """ Returns the names of all perfdata metrics that match selected request """
    metric_names = set()
    fields = "host_name description perf_data state host_state".split()
    services = adagios.status.utils.get_services(request, fields=fields, **kwargs)
    for service in services:
        metrics = pynag.Utils.PerfData(service['perf_data']).metrics
        metrics = filter(lambda x: x.is_valid(), metrics)
        for metric in metrics:
            metric_names.add(metric.label)

    result = {
        'services that match filter': len(services),
        'filter': kwargs,
        'metric_names': sorted(list(metric_names)),
    }
    return result

def wait(table, WaitObject, WaitCondition=None, WaitTrigger='check', **kwargs):
    print _("Lets wait for"), locals()
    if not WaitCondition:
        WaitCondition = "last_check > %s" % int(time.time()-1)
    livestatus = adagios.status.utils.livestatus(None)
    print _("livestatus ok")
    result = livestatus.get(table, 'Stats: state != 999', WaitObject=WaitObject, WaitCondition=WaitCondition, WaitTrigger=WaitTrigger, **kwargs)
    print _("ok no more waiting for "), WaitObject
    return result


def wait_many(hostlist, servicelist, WaitCondition=None, WaitTrigger='check', **kwargs):
    if not WaitCondition:
        WaitCondition = "last_check > %s" % int(time.time()-1)
    livestatus = adagios.status.utils.livestatus(None)
    for host in hostlist.split(';'):
        if not host:
            continue
        WaitObject = host
        livestatus.get('hosts', WaitObject=WaitObject, WaitCondition=WaitCondition, WaitTrigger=WaitTrigger, **kwargs)
        print WaitObject
    for service in servicelist.split(';'):
        if not service:
            continue
        WaitObject = service.replace(',', ';')
        livestatus.get('services', WaitObject=WaitObject, WaitCondition=WaitCondition, WaitTrigger=WaitTrigger, **kwargs)
        print WaitObject


def toggle_backend_visibility(request, backend_name):
    """ Toggles a backend in user preferences.

    Args:
      request: a Django request
      backend_name (str): The name of the backend.
    """
    user = userdata.User(request)
    if not user.disabled_backends:
        user.disabled_backends = []
    if backend_name in user.disabled_backends:
        user.disabled_backends.remove(backend_name)
    else:
        user.disabled_backends.append(backend_name)
    
    user.save()



# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.test import TestCase
from django.test.client import Client
from django.test.client import RequestFactory
import adagios.utils
import adagios.misc.rest
import adagios.userdata
import os


class FakeAdagiosEnvironment(TestCase):
    """ Test the features of adagios.utils.FakeAdagiosEnvironment
    """
    @classmethod
    def setUpClass(cls):
        cls.fake_adagios = adagios.utils.FakeAdagiosEnvironment()

    @classmethod
    def tearDownClass(cls):
        cls.fake_adagios.terminate()

    def testFakeAdagiosEnvironment(self):
        fake_adagios = self.fake_adagios

        # Make sure temporary environment gets created
        fake_adagios.create_minimal_environment()
        self.assertTrue(os.path.exists(fake_adagios.adagios_config_file))

        # Make sure adagios.settings is updated
        global_config_file = adagios.settings.adagios_configfile
        fake_adagios.update_adagios_global_variables()

        # Make sure adagios_config_file changed
        self.assertTrue(adagios.settings.adagios_configfile != global_config_file)

        # Make sure the new test is in the tempdir
        self.assertTrue(adagios.settings.adagios_configfile.startswith(fake_adagios.tempdir))

        # Make sure global variables are proparly restored
        fake_adagios.restore_adagios_global_variables()
        self.assertTrue(adagios.settings.adagios_configfile == global_config_file)


class MiscTestCase(TestCase):

    def setUp(self):
        self.environment = adagios.utils.get_test_environment()

    def tearDown(self):
        self.environment.terminate()

    def _testPageLoad(self, url):
        c = Client()
        response = c.get(url)
        self.assertEqual(response.status_code, 200)

    def TestPageLoads(self):
        """ Smoke test views in /misc/
        """
        self.loadPage("/misc/settings")
        self.loadPage("/misc/preferences")
        self.loadPage("/misc/nagios")
        self.loadPage("/misc/settings")
        self.loadPage("/misc/service")
        self.loadPage("/misc/pnp4nagios")
        self.loadPage("/misc/mail")
        self.loadPage("/misc/images")

    def loadPage(self, url):
        """ Load one specific page, and assert if return code is not 200 """
        try:
            c = Client()
            response = c.get(url)
            self.assertEqual(response.status_code, 200, _("Expected status code 200 for page %s") % url)
        except Exception, e:
            self.assertEqual(True, _("Unhandled exception while loading %(url)s: %(e)s") % {'url': url, 'e': e})

    def test_user_preferences(self):
        c = Client()
        response = c.post('/misc/preferences/',
                          {'theme': 'spacelab', 'language': 'fr'})

        assert(response.status_code == 200)
        assert('spacelab/style.css' in response.content)
        assert('(fr)' in response.content)

    def load_get(self, url):
        c = Client()
        response = c.get(url)
        return response

    # def test_topmenu_highlight(self):
    #     r = self.load_get('/status/')
    #     assert '<li class="active">\n<a href="/status">' in r.content

    # def test_leftmenu_highlight(self):
    #     r = self.load_get('/status/problems')
    #     assert '<li class="active">\n<a href="/status/problems">' in r.content

    def test_app_name(self):
        from adagios import settings
        settings.TOPMENU_HOME = 'Free beer'
        r = self.load_get('/status')
        assert 'Free beer' in r.content


class RestTest(TestCase):
    """Tests for adagios.misc.rest"""
    def setUp(self):
        self.environment = adagios.utils.get_test_environment()

        self.factory = RequestFactory()
        self.request = self.factory.get('/')
        self.user = adagios.userdata.User(self.request)
        self.user._username = 'test'
        self.user.__dict__ = {}
        self.user.save()

    def tearDown(self):
        self.environment.terminate()

    def test_save_search(self):
        adagios.misc.rest.save_search(self.request, 'test', 'http://localhost')
        user = adagios.userdata.User(self.request)
        self.assertEqual({'test':'http://localhost'}, user.to_dict()['saved_searches'])

    def test_get_saved_searches(self):
        adagios.misc.rest.save_search(self.request, 'test', 'http://adagios')
        self.assertEqual({'test':'http://adagios'}, adagios.misc.rest.get_saved_searches(self.request))

    def test_save_search_raises_on_no_name(self):
        with self.assertRaises(ValueError):
            adagios.misc.rest.save_search(self.request, '', 'http://localhost')

    def test_save_search_raises_on_no_url(self):
        with self.assertRaises(ValueError):
            adagios.misc.rest.save_search(self.request, 'name', '')

    def test_delete_saved_search(self):
        adagios.misc.rest.save_search(self.request, 'test', 'http://adagios')
        self.assertEqual({'test':'http://adagios'}, adagios.misc.rest.get_saved_searches(self.request))

        adagios.misc.rest.delete_saved_search(self.request, 'test')
        self.assertFalse(adagios.misc.rest.get_saved_searches(self.request))



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import url, patterns


urlpatterns = patterns('adagios',

    url(r'^/$', 'objectbrowser.views.list_object_types', name="objectbrowser"),

    url(r'^/edit_all/(?P<object_type>.+)/(?P<attribute_name>.+)/?$', 'objectbrowser.views.edit_all'),
    url(r'^/search/?$', 'objectbrowser.views.search_objects', name="search"),


    url(r'^/edit/(?P<object_id>.+?)?$', 'objectbrowser.views.edit_object', name="edit_object"),
    url(r'^/import/?$', 'objectbrowser.views.import_objects'),
    url(r'^/edit/?$', 'objectbrowser.views.edit_object'),
    url(r'^/copy_and_edit/(?P<object_id>.+?)?$', 'objectbrowser.views.copy_and_edit_object'),

    url(r'^/copy/(?P<object_id>.+)$', 'objectbrowser.views.copy_object', name="copy_object"),
    url(r'^/delete/(?P<object_id>.+)$', 'objectbrowser.views.delete_object', name="delete_object"),
    url(r'^/delete/(?P<object_type>.+?)/(?P<shortname>.+)/?$', 'objectbrowser.views.delete_object_by_shortname', name="delete_by_shortname"),

    url(r'^/add/(?P<object_type>.+)$', 'objectbrowser.views.add_object', name="addobject"),
    url(r'^/bulk_edit/?$', 'objectbrowser.views.bulk_edit', name='bulk_edit'),
    url(r'^/bulk_delete/?$', 'objectbrowser.views.bulk_delete', name='bulk_delete'),
    url(r'^/bulk_copy/?$', 'objectbrowser.views.bulk_copy', name='bulk_copy'),
    url(r'^/add_to_group/(?P<group_type>.+)/(?P<group_name>.+)/?$', 'objectbrowser.views.add_to_group'),
    url(r'^/add_to_group/(?P<group_type>.+)/?$', 'objectbrowser.views.add_to_group'),
    url(r'^/add_to_group', 'objectbrowser.views.add_to_group'),
    url(r'^/plugins/?$', 'objectbrowser.views.show_plugins'),
    url(r'^/nagios.cfg/?$', 'objectbrowser.views.edit_nagios_cfg'),
    url(r'^/nagios.cfg/edit/?$', 'misc.views.edit_nagios_cfg'),
    url(r'^/geek_edit/id=(?P<object_id>.+)$', 'objectbrowser.views.geek_edit'),
    url(r'^/advanced_edit/id=(?P<object_id>.+)$', 'objectbrowser.views.advanced_edit'),

    # Here for backwards compatibility.
    url(r'^/edit/id=(?P<object_id>.+)$', 'objectbrowser.views.edit_object', ),
    url(r'^/id=(?P<object_id>.+)$', 'objectbrowser.views.edit_object', ),

    # These should be deprecated as of 2012-08-27
    url(r'^/copy_object/id=(?P<object_id>.+)$', 'objectbrowser.views.copy_object'),
    url(r'^/delete_object/id=(?P<object_id>.+)$', 'objectbrowser.views.delete_object'),

    )



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import url, patterns

urlpatterns = patterns('adagios',
                       url(r'^/?$', 'rest.views.list_modules'),
                       )



# Example:
# rest_modules['module_name'] = 'module_path'
# will make /adagios/rest/module_name/ available and it loads all
# functions from 'module_path'

rest_modules = {}
rest_modules['pynag'] = 'adagios.misc.helpers'
rest_modules['okconfig'] = 'okconfig'
rest_modules['status'] = 'adagios.rest.status'
rest_modules['adagios'] = 'adagios.misc.rest'


# We are going to generate some url patterns, for clarification here is the end result shown for the status module:
#url(r'^/status/$', 'rest.views.index', { 'module_name': 'adagios.rest.status'    }, name="rest/status"),
#url(r'^/status.js$', 'rest.views.javascript', { 'module_name': 'adagios.rest.status'    }, ),
#(r'^/status/(?P<format>.+?)/(?P<attribute>.+?)/?$', 'rest.views.handle_request', { 'module_name': 'adagios.rest.status' }),

for module_name, module_path in rest_modules.items():
    base_pattern = r'^/%s' % module_name
    args = {'module_name': module_name, 'module_path': module_path}
    urlpatterns += patterns('adagios',
        url(base_pattern + '/$',   'rest.views.index', args, name="rest/%s" % module_name),
        url(base_pattern + '.js$', 'rest.views.javascript', args, ),
        url(base_pattern + '/(?P<format>.+?)/(?P<attribute>.+?)/?$', 'rest.views.handle_request', args),
    )



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Authorization related stuff in Adagios

"""

import adagios.status.utils
import adagios.views

auditors = []
operators = []
administrators = []

# administrator belongs to all the other groups
administrators += operators + auditors

access_list = list()

# Explicitly grant configuration access only to admins
access_list.append(('adagios.objectbrowser', "administrators"))
access_list.append(('adagios.okconfig_', "administrators"))
access_list.append(('adagios.misc.helpers', "administrators"))
access_list.append(('adagios.misc.views.settings', "administrators"))

access_list.append(('adagios.misc.views.gitlog', "administrators"))
access_list.append(('adagios.misc.views.service', "administrators"))
access_list.append(('adagios.rest.status.edit', "administrators"))
access_list.append(('adagios.status.views.contact', "administrators"))
access_list.append(('adagios.status.views.state_history', "administrators"))
access_list.append(('adagios.status.views.log', "administrators"))
access_list.append(('adagios.status.views.servicegroup', "administrators"))
access_list.append(('adagios.rest.status.state_history', "administrators"))
access_list.append(('adagios.rest.status.top_alert_producers', "administrators"))
access_list.append(('adagios.rest.status.update_check_command', "administrators"))
access_list.append(('adagios.rest.status.log_entries', "administrators"))



# Access to rest interface
access_list.append(('adagios.rest.views', "everyone"))
access_list.append(('adagios.rest.status', "everyone"))
access_list.append(('adagios.misc.rest', "everyone"))


# These modules should more or less be considered "safe"
access_list.append(('django.views.static', "everyone"))
access_list.append(('django.views.i18n', "everyone"))
access_list.append(('adagios.views', "everyone"))
access_list.append(('adagios.status', "everyone"))
access_list.append(('adagios.pnp', "everyone"))
access_list.append(('adagios.contrib', "everyone"))
access_list.append(('adagios.bi.views.index', "everyone"))
access_list.append(('adagios.bi.views.view', "everyone"))
access_list.append(('adagios.bi.views.json', "everyone"))
access_list.append(('adagios.bi.views.graphs_json', "everyone"))
access_list.append(('adagios.misc.helpers.needs_reload', "everyone"))


# If no other rule matches, assume administrators have access
access_list.append(('', "administrators"))


def check_access_to_path(request, path):
    """ Raises AccessDenied if user does not have access to path

    path in this case is a full path to a python module name for example: "adagios.objectbrowser.views.index"
    """
    for search_path, role in access_list:
        if path.startswith(search_path):
            if has_role(request, role):
                return None
            else:
                user = request.META.get('REMOTE_USER', 'anonymous')
                message = "You do not have permission to access %s" % (path, )
                raise adagios.exceptions.AccessDenied(user, access_required=role, message=message, path=path)
    else:
        return None


def has_access_to_path(request, path):
    """ Returns True/False if user in incoming request has access to path

     Arguments:
        path  -- string describing a path to a method or module, example: "adagios.objectbrowser.views.index"
    """
    for search_path, role in access_list:
        if path.startswith(search_path):
            return has_role(request, role)
    else:
        return False


def has_role(request, role):
    """ Returns true if the username in current request has access to a specific role """
    user = request.META.get('REMOTE_USER', "anonymous")

    # Allow if everyone is allowed access
    if role == 'everyone':
        return True

    # Deny if nobody is allowed access
    if role == 'nobody':
        return False

    # Allow if role is "contacts" and user is in fact a valid contact
    if role == 'contacts' and adagios.status.utils.get_contacts(None, name=user):
        return True

    # Allow if role is "users" and we are in fact logged in
    if role == 'users' and user != "anonymous":
        return True

    users_and_groups = globals().get(role, None)
    if hasattr(adagios.settings, role):
        for i in str(getattr(adagios.settings, role)).split(','):
            i = i.strip()
            if i not in users_and_groups:
                users_and_groups.append(i)


    # Deny if no role exists with this name
    if not users_and_groups:
        return False

    # Allow if user is mentioned in your role
    if user in users_and_groups:
        return True

    # If it is specifically stated that "everyone" belongs to the group
    if "everyone" in users_and_groups:
        return True

    # Check if user belongs to any contactgroup that has access
    contactgroups = adagios.status.utils.get_contactgroups(None, 'Columns: name', 'Filter: members >= %s' % user)

    # Allow if we find user belongs to one contactgroup that has this role
    for contactgroup in contactgroups:
        if contactgroup['name'] in users_and_groups:
            return True

    # If we get here, the user clearly did not have access
    return False


def check_role(request, role):
    """ Raises AccessDenied if user in request does not have access to role """
    if not has_role(request, role):
        user = request.META.get('REMOTE_USER', 'anonymous')
        message = "User does not have the required role"
        raise adagios.exceptions.AccessDenied(username=user, access_required=role, message=message)


class AuthorizationMiddleWare(object):
    """ Django MiddleWare class. It's responsibility is to check if an adagios user has access

    if user does not have access to a given view, it is given a 403 error.
    """
    def process_request(self, request):
        return None

    def process_view(self, request, view_func, view_args, view_kwargs):
        if not adagios.settings.enable_authorization:
            return None

        function_name = view_func.__name__
        module_name = view_func.__module__
        if module_name == "adagios.rest.views" and function_name == 'handle_request':
            module_name = view_kwargs['module_path']
            function_name = view_kwargs['attribute']

        try:
            path = module_name + '.' + function_name
            check_access_to_path(request, path)
        except adagios.exceptions.AccessDenied, e:
            return adagios.views.http_403(request, exception=e)



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from django.utils.translation import ugettext as _

def get_template_name(base_path, *args):
    """ Return a full path to a template named base_path/args[0]

    If multiple arguments are provided, treat them as recursive directories, but
    if a valid filename is found, return immediately.

    if base_path is not provided, use adagios.settings.contrib_dir
    """
    if not base_path:
        base_path = adagios.settings.contrib_dir
    path = base_path
    for i in args:
        if not i:
            continue
        path = os.path.join(path, i)
        path = os.path.normpath(path)

        if not path.startswith(base_path):
            raise Exception(_("'%(path)s' is outside contrib dir") % {'path': path})
        elif os.path.isdir(path):
            continue
        elif os.path.isfile(path):
            return path
    return path



#!/usr/bin/env python
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import adagios.status.utils
import adagios
import pynag.Model
import adagios.exceptions
import adagios.settings
import os
import pynag.Utils.misc
from multiprocessing.pool import ThreadPool
from django.utils.translation import ugettext as _

SELENIUM_DRIVER = None

def wait(object_type, WaitObject, WaitCondition, WaitTrigger, **kwargs):
    livestatus = adagios.status.utils.livestatus(None)
    livestatus.get(object_type, WaitObject=WaitObject, WaitCondition=WaitCondition, WaitTrigger=WaitTrigger, **kwargs)
    print WaitObject

def wait_for_objects(object_type, object_list, condition=None, trigger='check'):
    if not condition:
        condition = "last_check > %s" % int(0)
    callback = lambda x: wait(object_type, WaitObject=x, WaitCondition=condition, WaitTrigger=trigger)
    for WaitObject in object_list:
        callback(WaitObject)

def wait_for_service(host_name, service_description, condition='last_check >= 0', trigger='check'):
    livestatus = adagios.status.utils.livestatus(None)
    waitobject = "%s;%s" % (host_name, service_description)
    livestatus.get_services(
        host_name=host_name,
        service_description=service_description,
        WaitCondition=condition,
        WaitObject=waitobject
    )



class Task(object):
    def __init__(self, num_processes=5):
        self._tasks = []
        adagios.tasks.append(self)
        self._pool = ThreadPool(processes=num_processes)

    def add(self, function, *args, **kwargs):
        print "Adding Task:", locals()
        result = self._pool.apply_async(function, args, kwargs)
        self._tasks.append(result)
        #print result.get()

    def status(self):
        all_tasks = self._tasks
        for i in all_tasks:
            print i.ready()
        completed_tasks = filter(lambda x: x.ready(), all_tasks)
        return "{done}/{total} done.".format(done=len(completed_tasks), total=len(all_tasks))

    def get_id(self):
        return hash(self)

    def ready(self):
        """ Returns True if all the Tasks in this class have finished running. """
        return max(map(lambda x: x.ready(), self._tasks))


def update_eventhandlers(request):
    """ Iterates through all pynag eventhandler and informs them who might be making a change
    """
    remote_user = request.META.get('REMOTE_USER', 'anonymous')
    for i in pynag.Model.eventhandlers:
        i.modified_by = remote_user

    # if okconfig is installed, make sure okconfig is notified of git
    # settings
    try:
        from pynag.Utils import GitRepo
        import okconfig
        okconfig.git = GitRepo(directory=os.path.dirname(
            adagios.settings.nagios_config), auto_init=False, author_name=remote_user)
    except Exception:
        pass


def get_available_themes():
    """ Returns a tuple with the name of themes that are available in media/theme directory """
    theme_dir = os.path.join(adagios.settings.STATIC_ROOT, adagios.settings.THEMES_FOLDER)

    result = []
    for root, dirs, files in os.walk(theme_dir):
        if adagios.settings.THEME_ENTRY_POINT in files:
            result.append(os.path.basename(root))

    return result


class FakeAdagiosEnvironment(pynag.Utils.misc.FakeNagiosEnvironment):
    _adagios_settings_copy = None

    def __init__(self, *args, **kwargs):
        super(FakeAdagiosEnvironment, self).__init__(*args, **kwargs)

    def update_adagios_global_variables(self):
        """ Updates common adagios.settings to point to a temp directory.

         If you are are doing unit tests which require specific changes, feel free to update
         adagios.settings manually after calling this method.
        """
        self._adagios_settings_copy = adagios.settings.__dict__.copy()
        adagios.settings.adagios_configfile = self.adagios_config_file
        adagios.settings.USER_PREFS_PATH = self.adagios_config_dir + "/userdata"
        adagios.settings.nagios_config = self.cfg_file
        adagios.settings.livestatus_path = self.livestatus_socket_path
        adagios.settings.reload_configfile(self.adagios_config_file)


    def restore_adagios_global_variables(self):
        """ Restores adagios.settings so it looks like before update_adagios_global_variables() was called
        """
        adagios.settings.__dict__.clear()
        adagios.settings.__dict__.update(self._adagios_settings_copy)

    def create_minimal_environment(self):
        """ Behaves like FakeNagiosEnvironment except also creates adagios config directory """

        super(FakeAdagiosEnvironment, self).create_minimal_environment()
        self.adagios_config_dir = os.path.join(self.tempdir, 'adagios')
        self.adagios_config_file = os.path.join(self.adagios_config_dir, 'adagios.conf')

        os.makedirs(self.adagios_config_dir)
        with open(self.adagios_config_file, 'w') as f:
            f.write('')

    def terminate(self):
        """ Behaves like FakeNagiosEnvironment except also restores adagios.settings module """
        if self._adagios_settings_copy:
            self.restore_adagios_global_variables()
        super(FakeAdagiosEnvironment, self).terminate()

def get_test_environment():
    """Get a fake adagios environment for testing purposes.

    Convenvience method for getting and starting a fake nagios environment.

    Returns:
        FakeAdagiosEnvironment instance.
    """
    environment = FakeAdagiosEnvironment()
    environment.create_minimal_environment()
    environment.update_model()
    environment.update_adagios_global_variables()
    environment.configure_livestatus()
    return environment



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""
import tempfile
import os
import time




from django.test import TestCase
from django.test.client import Client
from django.utils.translation import ugettext as _

from adagios.bi import *
import adagios.utils


class TestBusinessProcess(TestCase):
    def setUp(self):
        fd, filename = tempfile.mkstemp()
        BusinessProcess._default_filename = filename

    def tearDown(self):
        os.remove(BusinessProcess._default_filename)

    def test_save_and_load(self):
        """ This test will test load/save of a business process.

         The procedure is as follows:
         * Load a business process
         * Save it
         * Make changes
         * Load it again, and verify changes were saved.
        """
        bp_name = 'test_business_process'

        b = BusinessProcess(bp_name)
        b.load()

        # Append a dot to the bp name and save
        new_display_name = b.display_name or '' + "."
        b.display_name = new_display_name
        b.save()

        # Load bp again
        b = BusinessProcess(bp_name)
        b.load()

        self.assertEqual(b.display_name, new_display_name)

    def test_add_process(self):
        """ Test adding new processes to a current BP
        """
        bp_name = 'test'
        sub_process_name = 'sub_process'
        sub_process_display_name = 'This is a subprocess of test'
        b = BusinessProcess(bp_name)
        b.add_process(sub_process_name, display_name=sub_process_display_name)
        for i in b.get_processes():
            if i.name == sub_process_name and i.display_name == sub_process_display_name:
                return
        else:
            self.assertTrue(
                False, 'We tried adding a business process but could not find it afterwards')

    def test_hostgroup_bp(self):
        bp_name = 'test'
        hostgroup_name = 'acme-network'
        b = BusinessProcess(bp_name)
        b.add_process(hostgroup_name, 'hostgroup')

    def test_remove_process(self):
        """ Test removing a subprocess from a businessprocess
        """
        bp_name = 'test'
        sub_process_name = 'sub_process'
        sub_process_display_name = 'This is a subprocess of test'
        b = BusinessProcess(bp_name)
        b.add_process(sub_process_name, display_name=sub_process_display_name)
        self.assertNotEqual([], b.processes)
        b.remove_process(sub_process_name)
        self.assertEqual([], b.processes)

    def test_get_all_processes(self):
        get_all_processes()

    def test_macros(self):
        bp = get_business_process('uniq test case', status_method="use_worst_state")

        macros_for_empty_process = {
            'num_problems': 0,
            'num_state_0': 0,
            'num_state_1': 0,
            'num_state_2': 0,
            'num_state_3': 0,
            'current_state': 3,
            'friendly_state': 'unknown',
            'percent_problems': 0,
            'percent_state_3': 0,
            'percent_state_2': 0,
            'percent_state_1': 0,
            'percent_state_0': 0
        }
        self.assertEqual(3, bp.get_status())
        self.assertEqual(macros_for_empty_process, bp.resolve_all_macros())

        bp.add_process("always_ok", status_method="always_ok")
        bp.add_process("always_major", status_method="always_major")

        macros_for_nonempty_process = {
            'num_problems': 1,
            'num_state_0': 1,
            'num_state_1': 0,
            'num_state_2': 1,
            'num_state_3': 0,
            'current_state': 2,
            'friendly_state': 'major problems',
            'percent_problems': 50.0,
            'percent_state_3': 0.0,
            'percent_state_2': 50.0,
            'percent_state_1': 0.0,
            'percent_state_0': 50.0
        }
        self.assertEqual(2, bp.get_status())
        self.assertEqual(macros_for_nonempty_process, bp.resolve_all_macros())

    def testPageLoad(self):
        self.loadPage('/bi')
        self.loadPage('/bi/add')
        self.loadPage('/bi/add/subprocess')
        self.loadPage('/bi/add/graph')

    def loadPage(self, url):
        """ Load one specific page, and assert if return code is not 200 """
        try:
            c = Client()
            response = c.get(url)
            self.assertEqual(response.status_code, 200, _("Expected status code 200 for page %s") % url)
        except Exception, e:
            self.assertEqual(True, "Unhandled exception while loading %s: %s" % (url, e))

    def test_delete(self):
        bp1 = BusinessProcess('bp1')
        bp1.save()

        bp2 = BusinessProcess('bp2')
        bp2.save()

        self.assertEqual(['bp1', 'bp2'], get_all_process_names())

    def test_delete_recursive(self):
        bp1 = BusinessProcess('bp1')
        bp2 = BusinessProcess('bp2')
        bp1.save()
        bp2.save()
        bp1.add_process('bp2', 'businessprocess')
        bp1.save()

        self.assertEqual(1, len(bp1.get_processes()))

        bp2.delete()
        bp1 = get_business_process('bp1')
        self.assertFalse(bp1.get_processes())


class TestBusinessProcessLogic(TestCase):
    """ This class responsible for testing business classes logic """
    def setUp(self):
        self.environment = adagios.utils.get_test_environment()
        self.addCleanup(self.environment.terminate)

        self.environment.start()

        fd, filename = tempfile.mkstemp()
        BusinessProcess._default_filename = filename

    def tearDown(self):
        os.remove(BusinessProcess._default_filename)

    def testBestAndWorstState(self):
        s = BusinessProcess("example process")
        s.status_method = 'use_worst_state'
        self.assertEqual(3, s.get_status(), _("Empty bi process should have status unknown"))

        s.add_process(process_name="always_ok", process_type="businessprocess", status_method='always_ok')
        self.assertEqual(0, s.get_status(), _("BI process with one ok subitem, should have state OK"))

        s.add_process("fail subprocess", status_method="always_major")
        self.assertEqual(2, s.get_status(), _("BI process with one failed item should have a critical state"))

        s.status_method = 'use_best_state'
        self.assertEqual(0, s.get_status(), _("BI process using use_best_state should be returning OK"))

    def testBusinessRules(self):
        s = BusinessProcess("example process")
        self.assertEqual(3, s.get_status(), _("Empty bi process should have status unknown"))

        s.add_process(process_name="always_ok", process_type="businessprocess", status_method='always_ok')
        self.assertEqual(0, s.get_status(), _("BI process with one ok subitem, should have state OK"))

        s.add_process("untagged process", status_method="always_major")
        self.assertEqual(0, s.get_status(), _("BI subprocess that is untagged should yield an ok state"))

        s.add_process("not critical process", status_method="always_major", tags="not critical")
        self.assertEqual(1, s.get_status(), _("A Non critical subprocess should yield 'minor problem'"))

        s.add_process("critical process", status_method="always_major", tags="mission critical")
        self.assertEqual(2, s.get_status(), _("A critical process in failed state should yield major problem"))

        s.add_process("another noncritical process", status_method="always_major", tags="not critical")
        self.assertEqual(2, s.get_status(), _("Adding another non critical subprocess should still yield a critical state"))


class TestDomainProcess(TestCase):
    """ Test the Domain business process type
    """
    def setUp(self):
        self.environment = adagios.utils.get_test_environment()
        self.addCleanup(self.environment.terminate)

        self.environment.start()

    def testHost(self):
        domain = get_business_process(process_name='ok.is', process_type='domain')

        # We don't exactly know the status of the domain, but lets run it anyway
        # for smoketesting
        domain.get_status()


class TestServiceProcess(TestCase):
    """ Test Service Business process type """
    def setUp(self):
        self.environment = adagios.utils.get_test_environment()
        self.addCleanup(self.environment.terminate)

        self.environment.start()

    def testService(self):
        service = get_business_process('ok_host/ok service 1', process_type='service')
        status = service.get_status()
        self.assertFalse(service.errors)
        self.assertEqual(0, status, "The service should always have status OK")


class TestHostProcess(TestCase):
    """ Test the Host business process type
    """
    def setUp(self):
        self.environment = adagios.utils.get_test_environment()
        self.addCleanup(self.environment.terminate)

        self.environment.start()

    def testNonExistingHost(self):
        host = get_business_process('non-existant host', process_type='host')
        self.assertEqual(3, host.get_status(), _("non existant host processes should have unknown status"))

    def testExistingHost(self):
        host = get_business_process('ok_host', process_type='host')
        self.assertEqual(0, host.get_status(), _("the host ok_host should always has status ok"))

    def testDomainProcess(self):
        # We don't exactly know the status of the domain, but lets run it anyway
        # for smoketesting
        get_business_process(process_name='oksad.is', process_type='domain')



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.http import HttpResponse
import traceback
from django.shortcuts import render_to_response, redirect
from django.template import RequestContext, loader
from django import template
from django.utils.translation import ugettext as _
import time
import logging
import adagios.settings
import adagios.utils
from adagios.exceptions import AccessDenied


def adagios_decorator(view_func):
    """ This is a python decorator intented for all views in the status module.

     It catches all unhandled exceptions and displays them on a generic web page.

     Kind of what the django exception page does when debug mode is on.
    """
    def wrapper(request, *args, **kwargs):
        start_time = time.time()
        try:
            if request.method == 'POST':
                adagios.utils.update_eventhandlers(request)
            result = view_func(request, *args, **kwargs)
            end_time = time.time()
            time_now = time.ctime()
            duration = end_time - start_time
            return result
        except Exception, e:
            c = {}
            c['exception'] = str(e)
            c['exception_type'] = str(type(e).__name__)
            c['traceback'] = traceback.format_exc()
            return error_page(request, context=c)
    wrapper.__name__ = view_func.__name__
    wrapper.__module__ = view_func.__module__
    return wrapper


def error_page(request, context=None):
    if context is None:
        context = {}
        context['errors'] = []
        context['errors'].append('Error occured, but no error messages provided, what happened?')
    if request.META.get('CONTENT_TYPE') == 'application/json':
        context.pop('request', None)
        content = str(context)
        response = HttpResponse(content=content, content_type='application/json')
    else:
        response = render_to_response('status_error.html', context, context_instance=RequestContext(request))
    response.status_code = 500
    return response


def index(request):
    """ This view is our frontpage """
    # If status view is enabled, redirect to frontpage of the status page:
    if adagios.settings.enable_status_view:
        return redirect('adagios.status.views.status_index', permanent=True)
    else:
        return redirect('objectbrowser', permanent=True)


def http_403(request, exception=None):
    context = {}
    context['exception'] = exception
    if request.META.get('CONTENT_TYPE') == 'application/json':
        c = {}
        c['exception_type'] = exception.__class__
        c['message'] = str(exception.message)
        c['access_required'] = exception.access_required
        response = HttpResponse(content=str(c), content_type='application/json')
    else:
        response = render_to_response('403.html', context, context_instance=RequestContext(request))
    response.status_code = 403
    return response



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django import forms
import okconfig
from adagios.misc import helpers
import re
from django.core.exceptions import ValidationError
import socket
from pynag import Model
from adagios.forms import AdagiosForm
from django.utils.translation import ugettext as _


def get_all_hosts():
    return [('', _('Select a host'))] + map(lambda x: (x, x), helpers.get_host_names())


def get_all_templates():
    all_templates = okconfig.get_templates()
    service_templates = filter(lambda x: 'host' not in x, all_templates)
    return map(lambda x: (x, _("Standard %(service_template)s checks") % {"service_template": x}), service_templates)


def get_all_groups():
    return map(lambda x: (x, x), okconfig.get_groups())


def get_inactive_services():
    """ List of all unregistered services (templates) """
    inactive_services = [('', _('Select a service'))]
    inactive_services += map(lambda x: (x.name, x.name),
                             Model.Service.objects.filter(service_description__contains="", name__contains="", register="0"))
    inactive_services.sort()
    return inactive_services


class ScanNetworkForm(AdagiosForm):
    network_address = forms.CharField()

    def clean_network_address(self):
        addr = self.cleaned_data['network_address']
        if addr.find('/') > -1:
            addr, mask = addr.split('/', 1)
            if not mask.isdigit():
                raise ValidationError(_("not a valid netmask"))
            if not self.isValidIPAddress(addr):
                raise ValidationError(_("not a valid ip address"))
        else:
            if not self.isValidIPAddress(addr):
                raise ValidationError(_("not a valid ip address"))
        return self.cleaned_data['network_address']

    def isValidHostname(self, hostname):
        if len(hostname) > 255:
            return False
        if hostname[-1:] == ".":
            # strip exactly one dot from the right, if present
            hostname = hostname[:-1]
        allowed = re.compile("(?!-)[A-Z\d-]{1,63}(?<!-)$", re.IGNORECASE)
        for x in hostname.split("."):
            if allowed.match(x) is False:
                return False
        return True

    def isValidIPAddress(self, ipaddress):
        try:
            socket.inet_aton(ipaddress)
        except Exception:
            return False
        return True


class AddGroupForm(AdagiosForm):
    group_name = forms.CharField(help_text=_("Example: databases"))
    alias = forms.CharField(help_text=_("Human friendly name for the group"))
    force = forms.BooleanField(
        required=False, help_text=_("Overwrite group if it already exists."))


class AddHostForm(AdagiosForm):
    host_name = forms.CharField(help_text=_("Name of the host to add"))
    address = forms.CharField(help_text=_("IP Address of this host"))
    group_name = forms.ChoiceField(
        initial="default", help_text=_("host/contact group to put this host in"))
    templates = forms.MultipleChoiceField(
        required=False, help_text=_("Add standard template of checks to this host"))
    force = forms.BooleanField(
        required=False, help_text=_("Overwrite host if it already exists."))

    def __init__(self, *args, **kwargs):
        super(self.__class__, self).__init__(*args, **kwargs)
        self.fields['group_name'].choices = choices = get_all_groups()
        self.fields['templates'].choices = get_all_templates()

    def clean(self):
        cleaned_data = super(AddHostForm, self).clean()
        force = self.cleaned_data.get('force')
        host_name = self.cleaned_data.get('host_name')
        templates = self.cleaned_data.get('templates')
        for i in templates:
            if i not in okconfig.get_templates().keys():
                self._errors['templates'] = self.error_class(
                    [_('template %s was not found') % i])
        if not force and host_name in okconfig.get_hosts():
            self._errors['host_name'] = self.error_class(
                [_('Host name already exists. Use force to overwrite')])
        return cleaned_data


class AddTemplateForm(AdagiosForm):
    # Attributes
    host_name = forms.ChoiceField(help_text=_("Add templates to this host"))
    templates = forms.MultipleChoiceField(
        required=False, help_text=_("Add standard template of checks to this host"))
    force = forms.BooleanField(
        required=False, help_text=_("Overwrites templates if they already exist"))

    def __init__(self, *args, **kwargs):
        super(self.__class__, self).__init__(*args, **kwargs)
        self.fields['templates'].choices = get_all_templates()
        self.fields['host_name'].choices = get_all_hosts()

    def clean(self):
        cleaned_data = super(AddTemplateForm, self).clean()
        force = self.cleaned_data.get('force')
        host_name = self.cleaned_data.get('host_name')
        templates = self.cleaned_data.get('templates')
        for i in templates:
            if i not in okconfig.get_templates().keys():
                self._errors['templates'] = self.error_class(
                    [_('template %s was not found') % i])
        if not force and host_name not in okconfig.get_hosts():
            self._errors['host_name'] = self.error_class(
                [_('Host name not found Use force to write template anyway')])
        return cleaned_data

    def save(self):
        host_name = self.cleaned_data['host_name']
        templates = self.cleaned_data['templates']
        force = self.cleaned_data['force']
        self.filelist = []
        for i in templates:
            self.filelist += okconfig.addtemplate(
                host_name=host_name, template_name=i, force=force)


class InstallAgentForm(AdagiosForm):
    remote_host = forms.CharField(help_text=_("Host or ip address"))
    install_method = forms.ChoiceField(
        initial='ssh', help_text=_("Make sure firewalls are not blocking ports 22(for ssh) or 445(for winexe)"),
        choices=[(_('auto detect'), _('auto detect')), ('ssh', 'ssh'), ('winexe', 'winexe')])
    username = forms.CharField(
        initial='root', help_text=_("Log into remote machine with as this user"))
    password = forms.CharField(
        required=False, widget=forms.PasswordInput, help_text=_("Leave empty if using kerberos or ssh keys"))
    windows_domain = forms.CharField(
        required=False, help_text=_("If remote machine is running a windows domain"))


class ChooseHostForm(AdagiosForm):
    host_name = forms.ChoiceField(help_text=_("Select which host to edit"))

    def __init__(self, service=Model.Service(), *args, **kwargs):
        super(forms.Form, self).__init__(*args, **kwargs)
        self.fields['host_name'].choices = get_all_hosts()


class AddServiceToHostForm(AdagiosForm):
    host_name = forms.ChoiceField(
        help_text=_("Select host which you want to add service check to"))
    service = forms.ChoiceField(
        help_text=_("Select which service check you want to add to this host"))

    def __init__(self, service=Model.Service(), *args, **kwargs):
        super(forms.Form, self).__init__(*args, **kwargs)
        self.fields['host_name'].choices = get_all_hosts()
        self.fields['service'].choices = get_inactive_services()


class EditTemplateForm(AdagiosForm):

    def __init__(self, service=Model.Service(), *args, **kwargs):
        self.service = service
        super(forms.Form, self).__init__(*args, **kwargs)

        # Run through all the all attributes. Add
        # to form everything that starts with "_"
        self.description = service['service_description']
        fieldname = "%s::%s::%s" % (
            service['host_name'], service['service_description'], 'register')
        self.fields[fieldname] = forms.BooleanField(
            required=False, initial=service['register'] == "1", label='register')
        self.register = fieldname

        macros = []
        self.command_line = None
        try:
            self.command_line = service.get_effective_command_line()
            for macro, value in service.get_all_macros().items():
                if macro.startswith('$_SERVICE') or macro.startswith('S$ARG'):
                    macros.append(macro)
            for k in sorted(macros):
                fieldname = "%s::%s::%s" % (
                    service['host_name'], service['service_description'], k)
                label = k.replace('$_SERVICE', '')
                label = label.replace('_', ' ')
                label = label.replace('$', '')
                label = label.capitalize()
                self.fields[fieldname] = forms.CharField(
                    required=False, initial=service.get_macro(k), label=label)
        # KeyError can occur if service has an invalid check_command
        except KeyError:
            pass

    def save(self):
        for i in self.changed_data:
            # Changed data comes in the format host_name::service_description::$_SERVICE_PING
            # We need to change that to just __PING
            field_name = i.split('::')[2]
            field_name = field_name.replace('$_SERVICE', '_')
            field_name = field_name.replace('$', '')
            data = self.cleaned_data[i]
            if field_name == 'register':
                data = int(data)
            self.service[field_name] = data
        self.service.save()
        self.service.reload_object()
        # Lets also update commandline because form is being returned to the
        # user
        self.command_line = self.service.get_effective_command_line()



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pynag.Model
import os
import getpass

from adagios import notifications, settings, add_plugin
from adagios.misc.rest import add_notification, clear_notification
from adagios.daemon import Daemon

import pynag.Model.EventHandlers
import pynag.Parsers
from pynag.Parsers import Livestatus
import adagios
import adagios.status.utils
from pynag import Model
import time
import datetime
from adagios import __version__
from adagios import userdata

from django.utils.translation import ugettext as _

def on_page_load(request):
    """ Collection of actions that take place every page load """
    results = {}
    update_global_variables()

    for k, v in reload_configfile(request).items():
        results[k] = v
    for k, v in get_httpuser(request).items():
        results[k] = v
    for k, v in get_tagged_comments(request).items():
        results[k] = v
    for k, v in check_nagios_running(request).items():
        results[k] = v
    for k, v in get_notifications(request).items():
        results[k] = v
    for k, v in get_unhandled_problems(request).items():
        results[k] = v
    for k, v in resolve_urlname(request).items():
        results[k] = v
    for k, v in check_selinux(request).items():
        results[k] = v
    for k, v in activate_plugins(request).items():
        results[k] = v
    for k, v in check_destination_directory(request).items():
        results[k] = v
    for k, v in check_nagios_cfg(request).items():
        results[k] = v
    for k, v in get_current_time(request).items():
        results[k] = v
    for k, v in get_okconfig(request).items():
        results[k] = v
    for k, v in get_nagios_url(request).items():
        results[k] = v
    for k, v in get_local_user(request).items():
        results[k] = v
    for k, v in get_current_settings(request).items():
        results[k] = v
    for k, v in get_plugins(request).items():
        results[k] = v
    for k, v in get_current_version(request).items():
        results[k] = v
    for k, v in get_serverside_includes(request).items():
        results[k] = v
    for k, v in get_user_preferences(request).items():
        results[k] = v
    for k, v in get_all_backends(request).items():
        results[k] = v
    for k, v in get_all_nonworking_backends(request).items():
        results[k] = v
    return results


def update_global_variables():
    """Updates all required global variables."""
    pynag.Model.cfg_file = adagios.settings.nagios_config


def get_current_time(request):
    """ Make current timestamp available to templates
    """
    result = {}
    try:
        now = datetime.datetime.now()
        result['current_time'] = now.strftime("%b %d %H:%M")
        result['current_timestamp'] = int(time.time())
    except Exception:
        return result
    return result


def get_serverside_includes(request):
    """ Returns a list of serverside includes to include on this page """
    result = {}
    try:
        result['ssi_headers'] = []
        result['ssi_footers'] = []
        dirname = adagios.settings.serverside_includes
        current_url = resolve_urlname(request).get('urlname')
        if not dirname:
            return {}
        if not os.path.isdir(dirname):
            return {}
        files = os.listdir(dirname)
        common_header_file = "common-header.ssi"
        common_footer_file = "common-footer.ssi"
        custom_header_file = "{urlname}-header.ssi".format(urlname=current_url)
        custom_footer_file = "{urlname}-footer.ssi".format(urlname=current_url)
        if common_header_file in files:
            result['ssi_headers'].append(dirname + "/" + common_header_file)
        if common_footer_file in files:
            result['ssi_footers'].append(dirname + "/" + common_footer_file)
        if custom_header_file in files:
            result['ssi_headers'].append(dirname + "/" + custom_header_file)
        if custom_footer_file in files:
            result['ssi_footers'].append(dirname + "/" + custom_footer_file)
    except Exception:
        return {}
    return result


def activate_plugins(request):
    """ Activates any plugins specified in settings.plugins """
    for k, v in settings.plugins.items():
        add_plugin(name=k, modulepath=v)
    return {'misc_menubar_items': adagios.misc_menubar_items, 'menubar_items': adagios.menubar_items}


def get_local_user(request):
    """ Return user that is running the adagios process under apache
    """
    user = getpass.getuser()
    return {'local_user': user}


def get_current_version(request):
    """ Returns current adagios version """
    return {'adagios_version': __version__}

def get_current_settings(request):
    """ Return a copy of adagios.settings
    """
    return {'settings': adagios.settings}


def resolve_urlname(request):
    """Allows us to see what the matched urlname for this
    request is within the template"""
    from django.core.urlresolvers import resolve
    try:
        res = resolve(request.path_info)
        if res:
            return {'urlname': res.url_name}
    except Exception:
        return {'urlname': 'None'}


def get_httpuser(request):
    """ Get the current user that is authenticating to us and update event handlers"""
    try:
        remote_user = request.META.get('REMOTE_USER', None)
    except Exception:
        remote_user = "anonymous"
    return {'remote_user': remote_user or "anonymous"}


def get_nagios_url(request):
    """ Get url to legasy nagios interface """
    return {'nagios_url': settings.nagios_url}


def get_tagged_comments(request):
    """ (for status view) returns number of comments that mention the remote_user"""
    try:
        remote_user = request.META.get('REMOTE_USER', 'anonymous')
        livestatus = adagios.status.utils.livestatus(request)
        tagged_comments = livestatus.query(
            'GET comments', 'Stats: comment ~ %s' % remote_user, columns=False)[0]
        if tagged_comments > 0:
            return {'tagged_comments': tagged_comments}
        else:
            return {'tagged_comments': ''}
    except Exception:
        return {'tagged_comments': ''}


def get_unhandled_problems(request):
    """ Get number of any unhandled problems via livestatus """
    results = {}
    try:
        livestatus = adagios.status.utils.livestatus(request)
        num_problems = livestatus.query('GET services',
                                        'Filter: state != 0',
                                        'Filter: acknowledged = 0',
                                        'Filter: host_acknowledged = 0',
                                        'Filter: scheduled_downtime_depth = 0',
                                        'Filter: host_scheduled_downtime_depth = 0',
                                        'Stats: state != 0',
                                        'Stats: host_state != 0',
                                        columns=False)
        results['num_problems'] = num_problems[0] + num_problems[1]
        results['num_unhandled_problems'] = num_problems[0] + num_problems[1]

        result = livestatus.query('GET services',
                                        'Stats: state != 0',
                                        'Stats: state != 0',
                                        'Stats: acknowledged = 0',
                                        'Stats: scheduled_downtime_depth = 0',
                                        'Stats: host_state = 0',
                                        'StatsAnd: 4',
                                        columns=False
                                        )
        num_service_problems_all = result[0]
        num_service_problems_unhandled = result[1]


        result = livestatus.query('GET hosts',
                                        'Stats: state != 0',
                                        'Stats: state != 0',
                                        'Stats: acknowledged = 0',
                                        'Stats: scheduled_downtime_depth = 0',
                                        'Stats: host_state = 1',
                                        'StatsAnd: 4',
                                        columns=False
                                        )
        num_host_problems_all = result[0]
        num_host_problems_unhandled = result[1]

        num_problems_all = num_service_problems_all + num_host_problems_all
        num_problems_unhandled = num_service_problems_unhandled + num_host_problems_unhandled

        num_problems = num_problems_unhandled

        results = locals()
        del results['livestatus']
        del results['result']
        del results['request']

    except Exception:
        pass
    return results


def check_nagios_cfg(request):
    """ Check availability of nagios.cfg """
    return {'nagios_cfg': pynag.Model.config.cfg_file}


def check_destination_directory(request):
    """ Check that adagios has a place to store new objects """
    dest = settings.destination_directory
    dest_dir_was_found = False

    # If there are problems with finding nagios.cfg, we don't
    # need to display any errors here regarding destination_directories
    try:
        Model.config.parse_maincfg()
    except Exception:
        return {}
    for k, v in Model.config.maincfg_values:
        if k != 'cfg_dir':
            continue
        if os.path.normpath(v) == os.path.normpath(dest):
            dest_dir_was_found = True
    if not dest_dir_was_found:
        add_notification(level="warning", notification_id="dest_dir",
                         message=_("Destination for new objects (%s) is not defined in nagios.cfg") % dest)
    elif not os.path.isdir(dest):
        add_notification(level="warning", notification_id="dest_dir",
                         message=_("Destination directory for new objects (%s) is not found. Please create it.") % dest)
    else:
        clear_notification(notification_id="dest_dir")
    return {}


def check_nagios_running(request):
    """ Notify user if nagios is not running """
    try:
        d = Daemon()
        return {"nagios_running": (d.running())}
    except Exception:
        return {}


def check_selinux(request):
    """ Check if selinux is enabled and notify user """
    notification_id = "selinux_active"
    if settings.warn_if_selinux_is_active:
        try:
            if open('/sys/fs/selinux/enforce', 'r').readline().strip() == "1":
                add_notification(
                    level="warning",
                    message=_('SELinux is enabled, which is likely to give your monitoring engine problems., see <a href="https://access.redhat.com/knowledge/docs/en-US/Red_Hat_Enterprise_Linux/6/html-single/Security-Enhanced_Linux/index.html#sect-Security-Enhanced_Linux-Enabling_and_Disabling_SELinux-Disabling_SELinux">here</a> for information on how to disable it.'),
                    notification_id=notification_id,
                )
        except Exception:
            pass
    else:
        clear_notification(notification_id)
    return {}


def get_notifications(request):
    """ Returns a hash map of adagios.notifications """
    return {"notifications": notifications}


def get_okconfig(request):
    """ Returns {"okconfig":True} if okconfig module is installed.
    """
    try:
        if "okconfig" in settings.plugins:
            return {"okconfig": True}
        return {}
    except Exception:
        return {}


def get_plugins(request):
    """
    """
    return {'plugins': settings.plugins}


def reload_configfile(request):
    """ Load the configfile from settings.adagios_configfile and put its content in adagios.settings. """
    try:
        clear_notification("configfile")
        settings.reload_configfile()
    except Exception, e:
        add_notification(
            level="warning", message=str(e), notification_id="configfile")
    return {}


def get_user_preferences(request):
    """ Loads the preferences for the logged-in user. """
    def theme_to_themepath(theme):
        return os.path.join(settings.THEMES_FOLDER,
                            theme,
                            settings.THEME_ENTRY_POINT)
    try:
        user = userdata.User(request)
        user.trigger_hooks()
        results = user.to_dict()
    except Exception:
        results = adagios.settings.PREFS_DEFAULT

    theme = results.get('theme', 'default')
    results['theme_path'] = theme_to_themepath(theme)
    return {'user_data': results}

def get_all_backends(request):
    backends = adagios.status.utils.get_all_backends()
    return {'backends': backends}

def get_all_nonworking_backends(request):
    """ Returns the backends which don't answer at the time. """
    b = [x for x in get_all_backends(request)['backends']
         if not Livestatus(x).test(raise_error=False)]
    return {'nonworking_backends': b}

if __name__ == '__main__':
    on_page_load(request=None)



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Exceptions that Adagios uses and raises
"""


class AdagiosError(Exception):
    """ Base Class for all Adagios Exceptions """
    pass


class AccessDenied(AdagiosError):
    """ This exception is raised whenever a user tries to access a page he does not have access to. """
    def __init__(self, username, access_required, message, path=None, *args, **kwargs):
        self.username = username
        self.access_required = access_required
        self.message = message
        self.path = path
        super(AccessDenied, self).__init__(message, *args, **kwargs)



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

import pynag.Utils
from pynag.Utils import PynagError
from adagios import settings
import subprocess

from django.utils.translation import ugettext as _


def run_pnp(pnp_command, **kwargs):
    """ Run a specific pnp command

    Arguments:
      pnp_command -- examples: image graph json xml export
      host        -- filter results for a specific host
      srv         -- filter results for a specific service
      source      -- Fetch a specific datasource (0,1,2,3, etc)
      view        -- Specific timeframe (0 = 4 hours, 1 = 25 hours, etc)
    Returns:
      Results as they appear from pnp's index.php
    Raises:
      PynagError if command could not be run

    """
    try:
        pnp_path = settings.pnp_path
    except Exception, e1:
        pnp_path = find_pnp_path()
    # Cleanup kwargs
    pnp_arguments = {}
    for k, v in kwargs.items():
        k = str(k)
        if isinstance(v, list):
            v = v[0]
        v = str(v)
        pnp_arguments[k] = v
    querystring = '&'.join(map(lambda x: "%s=%s" % x, pnp_arguments.items()))
    pnp_parameters = pnp_command + "?" + querystring
    command = ['php', pnp_path, pnp_parameters]
    proc = subprocess.Popen(command, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE,)
    stdout, stderr = proc.communicate('through stdin to stdout')
    result = proc.returncode, stdout, stderr
    return result[1]


def find_pnp_path():
    """ Look through common locations of pnp4nagios, tries to locate it automatically """
    possible_paths = [settings.pnp_filepath]
    possible_paths += [
        "/usr/share/pnp4nagios/html/index.php",
        "/usr/share/nagios/html/pnp4nagios/index.php"
    ]
    for i in possible_paths:
        if os.path.isfile(i):
            return i
    raise PynagError(
        _("Could not find pnp4nagios/index.php. Please specify it in adagios->settings->PNP. Tried %s") % possible_paths)



# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.test import TestCase
from django.test.client import Client
from django.utils.translation import ugettext as _
from django.core.urlresolvers import reverse
from django.http import QueryDict
import pynag.Model
import adagios.settings
import adagios.utils
import adagios.objectbrowser.forms
import re
import adagios.seleniumtests

from adagios.objectbrowser.forms import PynagAutoCompleteField

pynag.Model.cfg_file = adagios.settings.nagios_config

try:
    from selenium.webdriver.common.by import By
    from selenium.webdriver.support.ui import WebDriverWait
    from selenium.webdriver.support import expected_conditions as EC
    from selenium.common.exceptions import TimeoutException
except ImportError:
    # selenium tests are skipped if selenium is not available
    pass


class TestObjectBrowser(TestCase):

    def testNagiosConfigFile(self):
        result = pynag.Model.ObjectDefinition.objects.all
        config = pynag.Model.config.cfg_file
        self.assertGreaterEqual(
            len(result), 0, msg=_("Parsed nagios.cfg, but found no objects, are you sure this is the right config file (%(config)s) ? ") % {'config': config})

    def testIndexPage(self):
        c = Client()
        response = c.get('/objectbrowser/')
        self.assertEqual(response.status_code, 200)

    def testPageLoad(self):
        """ Smoke test a bunch of views """

        # TODO: Better tests, at least squeeze out a 200OK for these views
        self.loadPage('/objectbrowser/')
        self.loadPage('/objectbrowser/copy', 404)
        self.loadPage('/objectbrowser/search')
        self.loadPage('/objectbrowser/delete', 404)
        self.loadPage('/objectbrowser/bulk_edit')
        self.loadPage('/objectbrowser/bulk_delete')
        self.loadPage('/objectbrowser/bulk_copy')

        self.loadPage('/objectbrowser/edit_all', 404)
        self.loadPage('/objectbrowser/copy_and_edit', 301)

        self.loadPage('/objectbrowser/plugins')
        self.loadPage('/objectbrowser/nagios.cfg')
        self.loadPage('/objectbrowser/import')
        self.loadPage('/objectbrowser/geek_edit', 404)
        self.loadPage('/objectbrowser/advanced_edit', 404)

        #self.loadPage('/objectbrowser/add_to_group')
        self.loadPage('/objectbrowser/add/host', 200)
        self.loadPage('/objectbrowser/add/hostgroup', 200)
        self.loadPage('/objectbrowser/add/service', 200)
        self.loadPage('/objectbrowser/add/servicegroup', 200)
        self.loadPage('/objectbrowser/add/contact', 200)
        self.loadPage('/objectbrowser/add/contactgroup', 200)
        self.loadPage('/objectbrowser/add/timeperiod', 200)
        self.loadPage('/objectbrowser/add/command', 200)
        self.loadPage('/objectbrowser/add/template', 200)

    def loadPage(self, url, expected_code=200):
        """ Load one specific page, and assert if return code is not 200 """
        try:
            c = Client()
            response = c.get(url)
            self.assertEqual(response.status_code, expected_code, _("Expected status code 200 for page %(url)s") % {'url': url})
        except Exception, e:
            self.assertEqual(True, _("Unhandled exception while loading %(url)s: %(error)s") % {'url': url, 'error': e})


class TestPynagForm(TestCase):
    def setUp(self):
        self.nagios_config = adagios.settings.nagios_config
        self.environment = adagios.utils.get_test_environment()
        self.addCleanup(self.environment.terminate)

    def _create_new_host(self):
        host_name = 'unique foo'
        # Make sure host does not exist
        hosts = pynag.Model.Host.objects.filter(host_name=host_name)
        self.assertFalse(hosts, "There should be no host named %s" % host_name)
        host = pynag.Model.Host(host_name=host_name)
        host.save()

        # Make sure host was saved:
        hosts = pynag.Model.Host.objects.filter(host_name=host_name)
        self.assertTrue(hosts, "There should be one host named %s" % host_name)

        return hosts[0]

    def testBasicEdit(self):
        base_url = '/objectbrowser/edit/'

        host = self._create_new_host()
        host_name = host.host_name

        # Load the host via objectbrowser/edit
        url = '{base_url}{object_id}'
        url = url.format(base_url=base_url, object_id=host.get_id())

        c = Client()
        response = c.get(url)
        self.assertEqual(200, response.status_code)

        # See if the output more or less makes sense
        search_string_re = '<input[^>]* name="advanced-host_name"[^>]* value="{host_name}"'
        search_string_re = search_string_re.format(host_name=host_name)
        self.assertTrue(re.search(search_string_re, response.content))

        # Check the actual form we were sent
        form = response.context['form']
        data = form.initial

        self.assertIsNone(data.get('alias'))
        data['alias'] = host_name
        self.assertEqual(host_name, data.get('alias'))
        response = c.post(url, data=data)
        self.assertEqual(302, response.status_code)
        next_url = response['Location']

        response = c.get(next_url)
        self.assertEqual(200, response.status_code)
        form = response.context['form']
        data = form.initial
        self.assertEqual(data['host_name'], host_name)
        #TODO: Uncomment this once the underlying bug has been fixed
        #self.assertEqual(data['alias'], host_name)

    def testBasicEditing(self):
        host = self._create_new_host()
        host_name = host.host_name
        alias = 'some alias for this host'

        initial = host._original_attributes.copy()
        form = adagios.objectbrowser.forms.PynagForm(host, initial=initial)
        data = form.initial
        data['alias'] = alias

        form = adagios.objectbrowser.forms.PynagForm(host, data=data)
        self.assertTrue('alias' in form.data)
        self.assertEqual(alias, data['alias'])
        self.assertTrue('alias' in form.changed_data)
        self.assertTrue(form.is_valid())
        form.save()
        self.assertEqual(alias, form.pynag_object.alias)

        hosts = pynag.Model.Host.objects.filter(host_name=host_name)
        self.assertEqual(1, len(hosts), "there should be exactly 1 host with host_name %s" % host_name)
        host = hosts[0]
        self.assertEqual(host_name, host.host_name)
        self.assertEqual(alias, host.alias)

        # Test that there were no other attributes saved:
        attrs = sorted(host._defined_attributes.keys())
        self.assertEqual(['alias', 'host_name'], attrs)

    def testMultiChoiceFieldEmptyValue(self):
        host = self._create_new_host()
        hostgroups = ''
        host = self._change_object_via_form(host, hostgroups=hostgroups)
        self.assertIsNone(host.hostgroups)

        hostgroups = None
        host = self._change_object_via_form(host, hostgroups=hostgroups)
        self.assertIsNone(host.hostgroups)

        hostgroups = []
        host = self._change_object_via_form(host, hostgroups=hostgroups)
        self.assertIsNone(host.hostgroups)

        self._test_correct_attributes(host, 'host_name')

        hostgroups = 'bla,bla2'
        host = self._change_object_via_form(host, hostgroups=hostgroups)
        self.assertEqual('bla,bla2', host.hostgroups)

        self._test_correct_attributes(host, 'host_name', 'hostgroups')

    def _test_correct_attributes(self, pynag_object, *args):
        """ Asserts if pynag_object._defined_attributes does not match *args"""
        attrs = sorted(pynag_object._defined_attributes.keys())
        expected_attrs = sorted(args)
        self.assertEqual(expected_attrs, attrs)

    def _change_object_via_form(self, my_object=None, **kwargs):
        """ Changes a pynag object using PynagForm

        Args:
            my_object: any pynag object, if empty, autocreate new host
            **kwargs: Any kwargs provided will be attribute:new_value pairs
        Returns:
            The pynag object after it has been saved
        """
        if not my_object:
            my_object = self._create_new_host()

        data = {}
        for key, value in kwargs.iteritems():
            data[key] = value
        # Create new form for saving the data
        form = adagios.objectbrowser.forms.PynagForm(my_object, data=data)
        self.assertTrue(form.is_valid())
        form.save()

        new_id = form.pynag_object.get_id()
        new_object = pynag.Model.ObjectDefinition.objects.get_by_id(new_id)
        return new_object

    def test_changed_data(self):
        my_object = pynag.Model.Service()
        my_object.rewrite(_TEST_SERVICE)
        my_object.save()
        data = {}

        form = adagios.objectbrowser.forms.PynagForm(my_object, data=data)
        self.assertTrue(form.is_valid())
        self.assertFalse(form.changed_data)

        data['service_description'] = "new_description"
        form = adagios.objectbrowser.forms.PynagForm(my_object, data=data)
        self.assertTrue(form.is_valid())
        self.assertEqual(data.keys(), form.changed_data)

        data['hostgroup_name'] = 'bla'
        form = adagios.objectbrowser.forms.PynagForm(my_object, data=data)
        self.assertTrue(form.is_valid())
        self.assertListEqual(sorted(data.keys()), sorted(form.changed_data))

    def test_change_hostgroup(self):
        host = self._create_new_host()
        host_name = host.host_name
        hostgroups = '+h1,h2'
        host.hostgroups = hostgroups
        host.save()

        form = adagios.objectbrowser.forms.PynagForm(host)
        data = form.initial
        # Check we made no changes to the form and hostgroups is in fact
        # unchanged
        form = adagios.objectbrowser.forms.PynagForm(host, data=data)
        self.assertTrue(form.is_valid())
        form.save()
        host = pynag.Model.Host.objects.get_by_shortname(host_name)
        self.assertEqual(hostgroups, host.hostgroups)

        # Change the hostgroup and make sure the + is left intact
        form = adagios.objectbrowser.forms.PynagForm(host, data={'hostgroups': 'h2'})
        self.assertTrue(form.is_valid())
        form.save()
        host = pynag.Model.Host.objects.get_by_shortname(host_name)
        self.assertEqual('+h2', host.hostgroups)

    def test_issue_389_via_form(self):
        my_object = pynag.Model.Service()
        my_object.rewrite(_TEST_SERVICE)
        my_object.save()
        service_description = 'bla alias'

        new_object = self._change_object_via_form(my_object, service_description=service_description)
        self.assertEqual(service_description, new_object.service_description)

        data = QueryDict('check_period=24x7&use=generic-service&active_checks_enabled=0&notification_period=24x7&check_command=check-liebert-ac-temp&passive_checks_enabled=0&check_interval=1&service_description=Return+Temperatur&max_check_attempts=4&first_notification_delay=&csrfmiddlewaretoken=29180855d18882df3752806ed4d43567&notification_options=w&notification_options=c&notification_options=r&notification_options=u')
        form = adagios.objectbrowser.forms.PynagForm(new_object, data=data)
        self.assertTrue(form.is_valid())
        form.save()

        self.assertEqual('0', form.pynag_object.active_checks_enabled)
        self.assertEqual('HVAC', form.pynag_object.hostgroup_name)

    def test_issue_389_via_view(self):
        # In this test we have a problematic service that produces several bugs when user opens it in
        # objectbrowser and just presses save.
        my_object = pynag.Model.Service()
        my_object.rewrite(_TEST_SERVICE)
        my_object.save()

        url = '/objectbrowser/edit/{object_id}'.format(object_id=my_object.get_id())
        c = Client()

        # This is the exact querystring that was proplematic in issue 389. You can generate a new one
        # By editing edit_object() and printing out request.POST.urlencode()
        data = QueryDict('check_period=24x7&use=generic-service&active_checks_enabled=1&notification_interval=0&contacts=&notification_period=24x7&check_command=check-liebert-ac-temp&contact_groups=Facilities+Admin&servicegroups=&host_name=null&check_interval=1&service_description=Return+Temperature&csrfmiddlewaretoken=29180855d18882df3752806ed4d43567&first_notification_delay=&max_check_attempts=4&notification_options=w&notification_options=c&notification_options=r&notification_options=u&passive_checks_enabled=0')
        response = c.post(url, data)  # Simulate a user that just pressed save button.
        self.assertEqual(302, response.status_code)

        # Get the service again, verify it was changed
        services = pynag.Model.Service.objects.filter(service_description=my_object.service_description)
        self.assertEqual(
            1,
            len(services),
            "No service found with the new service description: %s" % my_object.service_description
        )
        new_service = services[0]

        # Manually change the servie description back, to original and make sure they are the same:
        new_service.service_description = my_object.service_description
        #self.assertEqual(my_object, new_service)

        # Iterate through every field in our original service, make sure
        # They are all same as expected
        for key, old_value in my_object._defined_attributes.items():
            new_value = new_service[key]
            # TODO: Find out what is broken in check_command
            if key in ('check_command'):
                continue
            message = "%s wasnt supposed to change after saving the service" % key
            self.assertEqual(old_value, new_value, message)

        # Same with the new object, just make sure nothinig got created that was not supposed to be there:
        for key in new_service._defined_attributes:
            # TODO: There is a bug which causes these extra attributes to be saved. Cant reproduce it in real
            # Life yet.
            if key in ('active_checks_enabled', 'notification_interval', 'max_check_attempts', 'notification_options'):
                continue
            message = "new service is not supposed to have %s after save" % key
            self.assertFalse(my_object[key] is None, message)

        self.assertTrue(new_service.hostgroup_name == 'HVAC')

    def test_init(self):

        dataset = {'host_name': 'new host_name'}
        empty_service = pynag.Model.Service()
        form = adagios.objectbrowser.forms.PynagForm(empty_service)
        self.assertEqual({}, form.initial)
        self.assertEqual({}, form.data)

        # Little field sanity check
        self.assertTrue('action_url' in form.fields)
        self.assertTrue('host_name' in form.fields)
        self.assertTrue('use' in form.fields)
        self.assertFalse('nonexistant' in form.fields)

        form = adagios.objectbrowser.forms.PynagForm(empty_service, initial=dataset)
        self.assertEqual(dataset, form.initial)
        self.assertEqual({}, form.data)

        form = adagios.objectbrowser.forms.PynagForm(empty_service, data=dataset)
        self.assertEqual(dataset, form.data)
        self.assertEqual({}, form.initial)

    def test_init_are_fields_initilized(self):
        dataset = {'host_name': 'new host_name'}
        empty_service = pynag.Model.Service()
        form = adagios.objectbrowser.forms.PynagForm(empty_service, initial=dataset)
        self.assertEqual(dataset, form.initial)
        self.assertEqual({}, form.data)

        for field_name, field in form.fields.items():
            expected_data = dataset.get(field_name, None)
            message = "Testing initial value for field %s" % field_name
            self.assertEqual(expected_data, field.initial, message)

    def test_initial_value_for_hostgroup(self):
        field_name = 'hostgroups'
        field_value = 'test,test2'
        clean_value = 'test,test2'

        dataset = {field_name: field_value}
        host = pynag.Model.Host(**dataset)
        empty_host = pynag.Model.Host()

        form = adagios.objectbrowser.forms.PynagForm(host)
        self.assertEqual(field_value, form.fields[field_name].initial)

        form = adagios.objectbrowser.forms.PynagForm(empty_host, initial=dataset)
        self.assertEqual(field_value, form.fields[field_name].initial)

        # Try same with a bound form
        form = adagios.objectbrowser.forms.PynagForm(empty_host, data=dataset)
        self.assertTrue(form.is_valid())
        self.assertTrue([field_name], form.changed_data)
        self.assertEqual(field_value, form.data[field_name])
        self.assertEqual(clean_value, form.cleaned_data[field_name])

    def test_initial_values(self):
        dataset = {'service_description': 'unit test', 'use': 'generic-service'}
        service = pynag.Model.Service(**dataset)
        form = adagios.objectbrowser.forms.PynagForm(service)
        self.assertEqual(dataset, form.initial)
        self.assertEqual({}, form.data)


class TestPynagAutoCompleteField(TestCase):
    def setUp(self):
        self.nagios_config = adagios.settings.nagios_config
        self.environment = adagios.utils.get_test_environment()
        self.addCleanup(self.environment.terminate)

    def test_init(self):
        field = PynagAutoCompleteField('host')
        self.assertIsInstance(field, adagios.objectbrowser.forms.PynagAutoCompleteField)

    def test_helptext(self):
        inline_help_text = 'this is a help text'
        field = adagios.objectbrowser.forms.PynagAutoCompleteField('host', inline_help_text=inline_help_text)
        self.assertEqual(inline_help_text, field.widget.attrs['data-placeholder'])

    def test_choices(self):
        field = PynagAutoCompleteField('host', complete="shortname")
        choices_string = field.widget.attrs['data-choices']
        choices_list = choices_string.split(',')
        self.assertIn('ok_host', choices_list)
        self.assertNotIn(None, choices_list)

        field = PynagAutoCompleteField('host', complete="name")
        choices_string = field.widget.attrs['data-choices']
        choices_list = choices_string.split(',')
        self.assertIn('generic-host', choices_list)
        self.assertIn('linux-server', choices_list)
        self.assertNotIn(None, choices_list)
        self.assertNotIn('apc02.disney.com', choices_list)

    def test_get_all_shortnames(self):
        object_type = 'host'
        field = adagios.objectbrowser.forms.PynagAutoCompleteField(object_type=object_type)
        shortnames = field.get_all_shortnames(object_type=object_type)
        self.assertIn('ok_host', shortnames)
        self.assertNotIn(None, shortnames)

    def test_get_all_object_names(self):
        object_type = 'host'
        field = adagios.objectbrowser.forms.PynagAutoCompleteField(object_type=object_type)
        names = field.get_all_object_names(object_type=object_type)
        self.assertNotIn(None, names)
        self.assertIn('generic-host', names)
        self.assertIn('linux-server', names)

    def test_prepare_value(self):
        field = PynagAutoCompleteField(object_type='host')

        self.assertEqual('a,b', field.prepare_value('a,b'))
        self.assertEqual('a,b', field.prepare_value('+a,b'))
        self.assertEqual('a', field.prepare_value('a'))
        self.assertEqual('null', field.prepare_value('null'))


class TestPynagChoiceField(TestCase):
    def setUp(self):
        self.nagios_config = adagios.settings.nagios_config
        self.environment = adagios.utils.get_test_environment()
        self.addCleanup(self.environment.terminate)

    def test_prepare_value(self):
        field = adagios.objectbrowser.forms.PynagChoiceField(initial=None)
        prepare_value = field.prepare_value

        self.assertEqual([], prepare_value(''))
        self.assertEqual([], prepare_value(None))
        self.assertEqual(['a'], prepare_value('a'))
        self.assertEqual(['a'], prepare_value('+a'))
        self.assertEqual(['a'], prepare_value('-a'))
        self.assertEqual(['a'], prepare_value('!a'))
        self.assertEqual(['a', 'b'], prepare_value('a,b'))
        self.assertEqual(['a', 'b'], prepare_value('+a,b'))
        self.assertEqual(['a', 'b'], prepare_value('!a, b'))

        # Test some invalid values
        with self.assertRaises(ValueError):
            prepare_value([])

    def test_init(self):
        initial = 'bla'
        inline_help_text = 'help me'
        field = adagios.objectbrowser.forms.PynagChoiceField(initial=initial, inline_help_text=inline_help_text)
        self.assertEqual(initial, field.initial)
        self.assertEqual(inline_help_text, field.widget.attrs.get('data-placeholder', None))

    def test_clean(self):
        field = adagios.objectbrowser.forms.PynagChoiceField(initial='+ab')
        clean = field.clean

        self.assertEqual('a,b', clean(['a', 'b']))
        self.assertEqual('a,b', clean(('a', 'b')))
        self.assertEqual('b,a', clean(('b', 'a')))

        field.set_prefix('+')
        self.assertEqual('+a,b', clean(['a', 'b']))

        self.assertEqual('null', field.clean([]))

    def test_set_prefix(self):
        field = adagios.objectbrowser.forms.PynagChoiceField()
        self.assertEqual('', field.get_prefix())

        field.set_prefix('+')
        self.assertEqual('+', field.get_prefix())

        field.set_prefix('')
        self.assertEqual('', field.get_prefix())


class TestImportObjectsForm(TestCase):
    def setUp(self):
        self.nagios_config = adagios.settings.nagios_config

        self.environment = adagios.utils.get_test_environment()
        self.addCleanup(self.environment.terminate)

        self.form = adagios.objectbrowser.forms.ImportObjectsForm()
        self.initial = {
            'objects': 'host_name,address\nlocalhost,127.0.0.1',
            'object_type': 'host',
            'seperator': ',',
            'destination_filename': '',
        }

    def test_is_valid_initial_is_false(self):
        self.assertEqual(False, self.form.is_valid())

    def test_is_valid_normal(self):
        form = adagios.objectbrowser.forms.ImportObjectsForm(data=self.initial)
        self.assertTrue(form.is_valid())

    def test_save(self):
        self.assertFalse(pynag.Model.Host.objects.filter(shortname='localhost'))
        form = adagios.objectbrowser.forms.ImportObjectsForm(data=self.initial)
        form.is_valid()
        objects = form.save()
        self.assertEqual(1, len(objects))
        self.assertTrue(pynag.Model.Host.objects.filter(shortname='localhost'))

class AddObjectForm(TestCase):

    def setUp(self):
        self.environment = adagios.utils.get_test_environment()
        self.addCleanup(self.environment.terminate)

    def test_get_template_if_it_exists(self):
        form = adagios.objectbrowser.forms.AddObjectForm('host')
        self.assertEqual(adagios.settings.default_host_template, form.get_template_if_it_exists())

    def test_get_template_if_it_exists_nonexistant(self):
        host = pynag.Model.Host.objects.get_by_name(adagios.settings.default_host_template)
        host.delete()
        form = adagios.objectbrowser.forms.AddObjectForm('host')
        self.assertEqual('', form.get_template_if_it_exists())

class SeleniumObjectBrowserTestCase(adagios.seleniumtests.SeleniumTestCase):
    def test_contacts_loading(self):
        """Test if contacts under configure loads"""
        for driver in self.drivers:
            driver.get(self.live_server_url + "/objectbrowser/#contact-tab_tab")

            wait = WebDriverWait(driver, 10)

            try:
                # Get all host rows
                contact_table_rows = wait.until(
                    EC.presence_of_all_elements_located((
                        By.XPATH,
                        "//table[contains(@id, 'contact-table')]/tbody/tr"))
                )
            except TimeoutException:
                self.assertTrue(False, "Timed out waiting for contact table to load")

            self.assertTrue(len(contact_table_rows) > 0,
                            "No table rows in contact table")

    def test_hosts_loading(self):
        """Test if hosts under configure loads"""
        for driver in self.drivers:
            driver.get(self.live_server_url + "/objectbrowser")

            wait = WebDriverWait(driver, 10)

            try:
                # Get all host rows
                host_table_rows = wait.until(
                    EC.presence_of_all_elements_located((
                        By.XPATH,
                        "//table[contains(@id, 'host-table')]/tbody/tr"))
                )
            except TimeoutException:
                self.assertTrue(False, "Timed out waiting for host table to load")

            self.assertTrue(len(host_table_rows) > 0,
                            "No table rows in host-table")

_TEST_SERVICE = """
define service {
    use generic-service
    check_command check-liebert-ac-temp!google123!returnTemp!"105,45"!"100,50"
    hostgroup_name HVAC
    passive_checks_enabled 0
    service_description Return Temperature
    host_name null
    check_interval 1
    contact_groups Facilities Admin
}
"""



# -*- coding: utf-8 -*-
#
# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Matthieu Caneill <matthieu.caneill@savoirfairelinux.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import json
import collections

import settings


class User(object):
    """ Handles authentified users, provides preferences management. """
    # Name of saved searches key
    SAVED_SEARCHES = 'saved_searches'
    def __init__(self, request, autosave=False):
        """ Instantiates one user's preferences.

        Args:
          request (Request): The incoming Django request.

        Kwargs:
          autosave (bool): if True, preferences are automatically saved.
        """
        self._request = request
        self._autosave = autosave
        try:
            self._username = request.META.get('REMOTE_USER', 'anonymous')
        except Exception:
            self._username = 'anonymous'
        self._conffile = self._get_prefs_location()
        self._check_path(self._conffile)
        # sets the preferences as attributes:
        for k, v in self._get_conf().iteritems():
            self.__dict__[k] = v

    def _check_path(self, path):
        """ Checks the userdata folder, try to create it if it doesn't
        exist."""
        folder = os.path.dirname(path)
        # does the folder exist?
        if not os.path.isdir(folder):
            try:
                os.makedirs(folder)
            except:
                raise Exception("Folder %s can't be created. Be sure Adagios "
                                "has write access on its parent." % folder)
    
    def _get_prefs_location(self):
        """ Returns the location of the preferences file of the
        specified user. """
        try:
            user_prefs_path = settings.USER_PREFS_PATH
        except:
            raise Exception('You must define USER_PREFS_PATH in settings.py')

        return os.path.join(user_prefs_path, self._username + '.json')

    def _get_default_conf(self):
        try:
            d = settings.PREFS_DEFAULT
        except:
            d = dict()
        return d
    
    def _get_conf(self):
        """ Returns the json preferences for the specified user. """
        try:
            with open(self._conffile) as f:
                conf = json.loads(f.read())
        except IOError:
            conf = self._get_default_conf()
        except ValueError:
            conf = self._get_default_conf()
        return conf
    
    def __getattr__(self, name):
        """ Provides None as a default value. """
        if name not in self.__dict__.keys():
            return None
        return self.__dict__[name]

    def __setattr__(self, name, value):
        """ Saves the preferences if autosave is set. """
        self.__dict__[name] = value
        if self._autosave and not name.startswith('_'):
            self.save()

    def set_pref(self, name, value):
        """ Explicitly sets a user preference. """
        self.__dict__[name] = value

    def to_dict(self):
        d = {}
        for k in filter(lambda x: not(x.startswith('_')), self.__dict__.keys()):
            d[k] = self.__dict__[k]
        return d
    
    def save(self):
        """ Saves  the preferences in JSON format. """
        d = self.to_dict()
        try:
            with open(self._conffile, 'w') as f:
                f.write(json.dumps(d))
        except IOError:
            raise Exception("Couldn't write settings into file %s. Be sure to "
                            "have write permissions on the parent folder."
                            % self._conffile)
        self.trigger_hooks()

    def trigger_hooks(self):
        """ Triggers the hooks when preferences are changed. """
        # language preference
        from django.utils import translation
        try:
            self._request.session['django_language'] = self.language
            # newer versions of Django: s/django_language/_language
            translation.activate(self.language)
        except Exception as e:
            pass



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models

# Create your models here.


class TestModel(models.Model):
    testField = models.CharField(max_length=100)
    testField2 = models.CharField(max_length=100)


class BusinessProcess(models.Model):
    processes = models.ManyToManyField("self", unique=False, blank=True)
    name = models.CharField(max_length=100, unique=True)
    display_name = models.CharField(max_length=100, blank=True)
    notes = models.CharField(max_length=1000, blank=True)
    #graphs = models.ManyToManyField(BusinessProcess, unique=False, blank=True)
    #graphs = models.ManyToManyField(BusinessProcess, unique=False, blank=True)


class Graph(models.Model):
    host_name = models.CharField(max_length=100)
    service_description = models.CharField(max_length=100)
    metric_name = models.CharField(max_length=100)



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os


def startup():
    """ Do some pre-loading and parsing of objects

    Returns:
        None
    """
    from adagios import settings

    pynag.Model.cfg_file = settings.nagios_config
    pynag.Model.pynag_directory = settings.destination_directory

    # Pre load objects on startup
    #pynag.Model.ObjectDefinition.objects.get_all()

    from pynag.Model import EventHandlers
    if settings.enable_githandler == True:
        nagios_dir = os.path.dirname(pynag.Model.config.cfg_file)
        githandler = pynag.Model.EventHandlers.GitEventHandler(nagios_dir, 'adagios', 'tommi')
        pynag.Model.eventhandlers.append(githandler)
    if settings.auto_reload:
        handler = pynag.Model.EventHandlers.NagiosReloadHandler(nagios_init=settings.nagios_init_script)
        pynag.Model.eventhandlers.append(handler)

# If any pynag errors occur during initial parsing, we ignore them
# because we still want the webserver to start.
# Any errors that might occur will happen on page load anyway
# so the user will be informed
try:
    import pynag.Model
    startup()
except Exception, e:
    pass



# Adagios is a web based Nagios configuration interface
#
# Copyright (C) 2014, Pall Sigurdsson <palli@opensource.is>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""

from django.test import TestCase


class SimpleTest(TestCase):
    def test_basic_addition(self):
        """
        Tests that 1 + 1 always equals 2.
        """
        self.assertEqual(1 + 1, 2)
